@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model;
-$Config::Model::VERSION = '2.097';
+$Config::Model::VERSION = '2.117';
 use strict ;
 use warnings;
 use 5.10.1;
@@ -23,7 +23,8 @@ use Data::Dumper ();
 use Log::Log4perl 1.11 qw(get_logger :levels);
 use Config::Model::Instance;
 use Hash::Merge 0.12 qw/merge/;
-use File::Path qw/make_path/;
+use Path::Tiny 0.053;
+use File::HomeDir;
 
 use Cwd;
 use Config::Model::Lister;
@@ -35,6 +36,7 @@ our @EXPORT_OK = qw/cme/;
 use vars qw(@status @level %default_property);
 
 my $legacy_logger = get_logger("Model::Legacy") ;
+my $loader_logger = get_logger("Model::Loader") ;
 my $logger = get_logger("Model") ;
 
 # used to keep one Config::Model object to simplify programs based on
@@ -126,6 +128,27 @@ has models => (
     },
 );
 
+# model snippet may be loaded when the target class is not available
+# so they must be stored before being used.
+has model_snippets => (
+    isa     => 'ArrayRef',
+    is      => 'ro',
+    default => sub { [] },
+    traits  => ['Array'],
+    handles => {
+        add_snippet => 'push',
+        all_snippets => 'elements',
+    },
+);
+
+
+enum 'LOG_LEVELS', [ qw/ERROR WARN INFO DEBUG TRACE/ ];
+
+has log_level => (
+    isa => 'LOG_LEVELS',
+    is => 'ro',
+);
+
 has skip_inheritance => (
     isa     => 'Bool',
     is      => 'ro',
@@ -141,7 +164,17 @@ around BUILDARGS => sub {
     my $class = shift;
 
     my %args = @_;
-    my %new = map { defined $args{$_} ? ( $_ => $args{$_} ) : () } keys %args;
+    my %new;
+    foreach my $k (keys %args) {
+        if (defined $args{$k}) {
+            $new{$k} = $args{$k};
+        }
+        else {
+            # this warning should be changed to an error end of 2017
+            # cannot use logger, it's not initialised yet
+            warn("Config::Model new: passing undefined constructor argument is deprecated ($k argument)\n");
+        }
+    }
 
     return $class->$orig(%new);
 };
@@ -149,29 +182,55 @@ around BUILDARGS => sub {
 # keep this as a separate sub from BUILD. So user can call it before
 # creating Config::Model object
 sub initialize_log4perl {
-    my $log4perl_syst_conf_file = '/etc/log4config-model.conf';
-    my $log4perl_user_conf_file = $ENV{HOME} . '/.log4config-model';
+    my $self = shift;
+    my $args = shift;
+
+    my $log4perl_syst_conf_file = path('/etc/log4config-model.conf');
+    # avoid undef warning when homedir is not defined (e.g. with Debian cowbuilder)
+    my $home = File::HomeDir->my_home // '';
+    my $log4perl_user_conf_file = path( $home . '/.log4config-model' );
 
     my $fallback_conf           = << 'EOC';
-log4perl.logger=WARN, Screen
+log4perl.rootLogger=WARN, Screen
+
+# user message about deprecation issues
+log4perl.logger.Model.Legacy = INFO, SimpleScreen
+log4perl.additivity.Model.Legacy = 0
+
+# messages for users, aims to replace calls to warn for warnings or fix messages
+log4perl.logger.User = WARN, SimpleScreen
+log4perl.additivity.User = 0
+
 log4perl.appender.Screen        = Log::Log4perl::Appender::Screen
 log4perl.appender.Screen.stderr = 0
 log4perl.appender.Screen.layout = Log::Log4perl::Layout::PatternLayout
-log4perl.appender.Screen.layout.ConversionPattern = %d %m %n
+log4perl.appender.Screen.layout.ConversionPattern = %M %m (line %L)%n
+
+log4perl.appender.SimpleScreen        = Log::Log4perl::Appender::Screen
+log4perl.appender.SimpleScreen.stderr = 1
+log4perl.appender.SimpleScreen.layout = Log::Log4perl::Layout::PatternLayout
+log4perl.appender.SimpleScreen.layout.ConversionPattern = %p: %m%n
+
+log4perl.oneMessagePerAppender = 1
 EOC
 
-    my $log4perl_conf =
-        -e $log4perl_user_conf_file ? $log4perl_user_conf_file
-      : -e $log4perl_syst_conf_file ? $log4perl_syst_conf_file
-      :                               \$fallback_conf;
+    my @log4perl_conf_lines =
+        $log4perl_user_conf_file->is_file ? $log4perl_user_conf_file->lines
+      : $log4perl_syst_conf_file->is_file ? $log4perl_syst_conf_file->lines
+      :                                     split /\n/, $fallback_conf;
+    my %log4perl_conf = map { split /\s*=\s*/,$_,2; } grep { chomp; ! /^\s*#/ } @log4perl_conf_lines;
 
-    Log::Log4perl::init_once($log4perl_conf);
+    if (defined $args->{log_level}) {
+        $log4perl_conf{'log4perl.logger'} = $args->{log_level}.', Screen';
+    }
+
+    Log::Log4perl::init(\%log4perl_conf);
 
 }
 
 sub BUILD {
     my $self = shift;
-    $self->initialize_log4perl ;
+    $self->initialize_log4perl(shift) unless Log::Log4perl->initialized();
 }
 
 sub show_legacy_issue {
@@ -180,13 +239,14 @@ sub show_legacy_issue {
     my $behavior = shift || $self->legacy;
 
     my @msg = ref $ref ? @$ref : $ref;
+    unshift @msg, "Model ";
     if ( $behavior eq 'die' ) {
         die @msg, "\n";
     }
     elsif ( $behavior eq 'warn' ) {
-        warn @msg, "\n";
+        $legacy_logger->warn(@msg);
     } elsif ( $behavior eq 'note' ) {
-        say @msg;
+        $legacy_logger->info( @msg);
     }
 }
 
@@ -197,10 +257,18 @@ sub _tweak_instance_args {
     my $cat = '';
     if (defined $application) {
         my ( $categories, $appli_info, $appli_map ) = Config::Model::Lister::available_models;
-        $args->{root_class_name} ||= $appli_map->{$application} ;
+
+        # root_class_name may override class found (or not) by appli in tests
+        if (not $args->{root_class_name}) {
+            $args->{root_class_name} = $appli_map->{$application} ||
+                die "Unknown application $application. Expected one of "
+                . join(' ',sort keys %$appli_map)."\n";
+        }
+
         $cat = $appli_info->{_category} //  ''; # may be empty in tests
         # config_dir may be specified in application file
         $args->{config_dir} //= $appli_info->{$application}{config_dir};
+        $args->{appli_info} = $appli_info->{$application} // {};
     }
 
     my $app_name = $application;
@@ -225,7 +293,6 @@ sub cme {
     # model_storage is used to keep Config::Model object alive
     $model_storage //= Config::Model->new(%$m_args);
 
-    my $app = $args{application};
     return $model_storage->instance(@_);
 }
 
@@ -278,13 +345,12 @@ sub instance_names {
 # into element description.
 
 my @legal_params_to_move = (
-    qw/read_config  read_config_dir
-       write_config write_config_dir/,    # read/write stuff
+    qw/read_config write_config rw_config/,    # read/write stuff
 
     # this parameter is filled by class generated by a program. It may
     # be used to avoid interactive edition of a generated model
     'generated_by',
-    qw/class_description author copyright license include include_after include_backend class/
+    qw/class_description author copyright gist license include include_after include_backend class/
 );
 
 my @other_legal_params = qw/ author element status description summary level accept/;
@@ -360,7 +426,7 @@ sub include_backend {
         # includes (and normalization). Is already a dclone
         my $included_model = $self->get_model($included_class);
 
-        foreach my $rw (qw/read_config write_config config_dir/) {
+        foreach my $rw (qw/rw_config read_config write_config config_dir/) {
             if ($target_model->{$rw} and $included_model->{$rw}) {
                 my $msg = "Included $rw from $included_class cannot clobber "
                     . "existing data in $class_name";
@@ -428,15 +494,14 @@ sub normalize_class_parameters {
         $normalized_model->{include} = delete $normalized_model->{inherit};
     }
 
-    # get data read/write information (if any)
-    $model->{read_config_dir} = $model->{write_config_dir} = delete $normalized_model->{config_dir}
-        if defined $normalized_model->{config_dir};
-
     foreach my $info (@legal_params_to_move) {
         next unless defined $normalized_model->{$info};
         $model->{$info} = delete $normalized_model->{$info};
     }
 
+    # first deal with perl file and cds_file backend
+    $self->translate_legacy_backend_info( $config_class_name, $model );
+
     # handle accept parameter
     my @accept_list;
     my %accept_hash;
@@ -632,6 +697,58 @@ sub translate_legacy_info {
     ) if $legacy_logger->is_debug;
 }
 
+sub translate_legacy_backend_info {
+    my ( $self, $config_class_name, $model ) = @_;
+
+    # trap multi backend and change array spec into single spec
+    foreach my $config (qw/read_config write_config/) {
+        my $ref = $model->{$config};
+        if ($ref and ref($ref) eq 'ARRAY') {
+            if (@$ref == 1) {
+                $model->{$config} = $ref->[0];
+            }
+            elsif (@$ref > 1){
+                $self->show_legacy_issue("$config_class_name $config: multiple backends are obsolete. You now must use only one backend.", 'die');
+            }
+        }
+    }
+
+    # move read_config spec in re_config
+    if ($model->{read_config}) {
+        $self->show_legacy_issue("$config_class_name: read_config specification is deprecated, please move in rw_config", 'warn');
+        $model->{rw_config} = delete $model->{read_config};
+    }
+
+    # merge write_config spec in rw_config
+    if ($model->{write_config}) {
+        $self->show_legacy_issue("$config_class_name: write_config specification is deprecated, please merge with read_config and move in rw_config", 'warn');
+        map {$model->{rw_config}{$_} = $model->{write_config}{$_} } keys %{$model->{write_config}} ;
+        delete $model->{write_config};
+    }
+
+    my $ref = $model->{'rw_config'} || return;
+
+    die "undefined backend in rw_config spec of class $config_class_name\n" unless $ref->{backend} ;
+
+    if ($ref->{backend} eq 'custom') {
+        my $msg = "$config_class_name: custom read/write backend is obsolete."
+            ." Please replace with a backend inheriting Config::Model::Backend::Any";
+        $self->show_legacy_issue( $msg, 'die');
+    }
+
+    if ( $ref->{backend} =~ /^(perl|ini|cds)$/ ) {
+        my $backend = $ref->{backend};
+        $self->show_legacy_issue("$config_class_name: deprecated backend '$backend'. Should be '$ {backend}_file'", 'warn');
+        $ref->{backend} .= "_file";
+    }
+
+    if ( defined $ref->{allow_empty} ) {
+        $self->show_legacy_issue("$config_class_name: backend $ref->{backend}: allow_empty is deprecated. Use auto_create", 'warn');
+        $ref->{auto_create} = delete $ref->{allow_empty};
+    }
+
+}
+
 sub translate_cargo_info {
     my $self              = shift;
     my $config_class_name = shift;
@@ -666,15 +783,14 @@ sub translate_cargo_info {
     ) if $legacy_logger->is_debug;
 }
 
-# TODO: set to die In September 2016
 sub translate_id_names {
     my $self              = shift;
     my $config_class_name = shift;
     my $elt_name          = shift;
     my $info              = shift;
-    $self->translate_name( $config_class_name, $elt_name, $info, 'allow',      'allow_keys',       'warn' );
-    $self->translate_name( $config_class_name, $elt_name, $info, 'allow_from', 'allow_keys_from',  'warn' );
-    $self->translate_name( $config_class_name, $elt_name, $info, 'follow',     'follow_keys_from', 'warn' );
+    $self->translate_name( $config_class_name, $elt_name, $info, 'allow',      'allow_keys',       'die' );
+    $self->translate_name( $config_class_name, $elt_name, $info, 'allow_from', 'allow_keys_from',  'die' );
+    $self->translate_name( $config_class_name, $elt_name, $info, 'follow',     'follow_keys_from', 'die' );
 }
 
 sub translate_name {
@@ -1178,47 +1294,76 @@ sub load {
     my $model_name = shift;    # model name like Foo::Bar
     my $load_file  = shift;    # model file (override model name), used for tests
 
+    $loader_logger->debug("called on model $model_name");
+
     my $load_path = $model_name;
     $load_path =~ s/::/\//g;
 
     $load_file ||= $self->model_dir . '/' . $load_path . '.pl';
 
-    get_logger("Model::Loader")->debug("model $model_name from file $load_file");
+    $loader_logger->debug("model $model_name from file $load_file");
 
-    # no special treatment, returns an array
     my %models_by_name;
+
+    # Searches $load_file in @INC and returns an array containing the
+    # names of the loaded clases
     my @loaded_classes = $self->_load_model_in_hash( \%models_by_name, $load_file );
+
     $self->store_raw_model( $model_name, dclone( \%models_by_name ) );
+
     foreach my $name ( keys %models_by_name ) {
         my $data = $self->normalize_class_parameters( $name, $models_by_name{$name} );
-        get_logger("Model::Loader")->debug("Store normalized model $name");
+        $loader_logger->debug("Store normalized model $name");
         $self->store_normalized_model( $name, $data );
     }
 
     # look for additional model information
     my %model_graft_by_name;
     my %done;  # avoid loading twice the same snippet (where system version may clobber dev version)
-    foreach my $inc (@INC,'.') {
+
+    foreach my $inc_str (@INC) {
         foreach my $name ( keys %models_by_name ) {
             my $snippet_path = $name;
             $snippet_path =~ s/::/\//g;
-            my $snippet_dir = "$inc/" . $self->model_dir . '/' . $snippet_path . '.d';
-            get_logger("Model::Loader")->trace("looking for snippet in $snippet_dir");
-            if ( -d $snippet_dir ) {
-                foreach my $snippet_file ( glob("$snippet_dir/*.pl") ) {
-                    my $done_key = $name . ':' . $snippet_file;
+            my $snippet_dir = path($inc_str)->child($self->model_dir)->child($snippet_path . '.d');
+            $loader_logger->trace("looking for snippet in $snippet_dir");
+            if ( $snippet_dir->is_dir ) {
+                my $iter = $snippet_dir->iterator({ recurse => 1 });
+
+                while ( my $snippet_file = $iter->() ) {
+                    next unless $snippet_file =~ /\.pl$/;
+
+                    # $snippet_file (Path::Tiny object) was
+                    # constructed from @INC content (i.e. $inc_str)
+                    # and contains an absolute path. Since
+                    # _load_model_in_hash uses 'do' (which may search
+                    # in @INC), the file path passed to
+                    # _load_model_in_hash must be either absolute or
+                    # relative to $inc_str
+                    my $snippet_file_rel = $snippet_file->relative($inc_str);
+
+                    my $done_key = $name . ':' . $snippet_file_rel;
                     next if $done{$done_key};
-                    get_logger("Model::Loader")->info("Found snippet $snippet_file");
-                    $self->_load_model_in_hash( \%model_graft_by_name, $snippet_file );
+                    $loader_logger->info("Found snippet $snippet_file_rel in $inc_str dir");
+                    $self->_load_model_in_hash( \%model_graft_by_name, $snippet_file_rel);
                     $done{$done_key} = 1;
                 }
             }
         }
     }
 
-    foreach my $class_to_merge ( keys %model_graft_by_name ) {
-        my $data = $model_graft_by_name{$class_to_merge};
-        $self->augment_config_class_really( $class_to_merge, $data );
+    # store snippet. May be used later
+    foreach my $name (keys %model_graft_by_name) {
+        # store snippet for later usage
+        $loader_logger->trace("storing snippet for model $name");
+        $self->add_snippet($model_graft_by_name{$name});
+    }
+
+    # check if a snippet is available for this class
+    foreach my $snippet ( $self->all_snippets ) {
+        my $class_to_merge = $snippet->{name};
+        next unless $models_by_name{$class_to_merge};
+        $self->augment_config_class_really( $class_to_merge, $snippet );
     }
 
     # return the list of classes found in $load_file. Respecting the order of the class
@@ -1257,10 +1402,10 @@ sub _load_model_in_hash {
 sub _do_model_file {
     my ( $self, $load_file ) = @_;
 
-    get_logger("Model::Loader")->info("load model $load_file");
+    $loader_logger->info("load model $load_file");
 
     my $err_msg = '';
-    $load_file = "./$load_file" if $load_file !~ m!^/! and -e $load_file ;
+    # do searches @INC if the file path is not absolute
     my $model   = do $load_file;
 
     unless ($model) {
@@ -1296,7 +1441,7 @@ sub augment_config_class_really {
     my $orig_model = $self->normalized_model($config_class_name);
     croak "unknown class to augment: $config_class_name" unless defined $orig_model;
 
-    my $model_addendum = $self->normalize_class_parameters( $config_class_name, $augment_data );
+    my $model_addendum = $self->normalize_class_parameters( $config_class_name, dclone($augment_data) );
 
     my $merge = Hash::Merge->new('RIGHT_PRECEDENT');
     my $new_model = $merge->merge( $orig_model, $model_addendum );
@@ -1325,7 +1470,7 @@ sub get_model {
         unless $self->normalized_model_exists($config_class_name);
 
     if ( not $self->model_defined($config_class_name) ) {
-        get_logger("Model::Loader")->debug("creating model $config_class_name");
+        $loader_logger->debug("creating model $config_class_name");
 
         my $model = $self->merge_included_class($config_class_name);
         $self->_store_model( $config_class_name, $model );
@@ -1364,7 +1509,7 @@ sub get_model_doc {
 
             # Pod::Weaver compatibility
             "# PODNAME: $full_name",
-            "# ABSTRACT:  Configuration class " . $class_name,
+            "# ABSTRACT:  Configuration class " . $class_name, '',
 
             # assume utf8 for all docs
             "=encoding utf8", '',
@@ -1403,6 +1548,9 @@ sub get_model_doc {
                 if ( my $migr = $_->{migrate_values_from} ) {
                     push @elt, "Note: $elt_name values are migrated from '$migr'", '';
                 }
+                if ( my $comp = $_->{compute} ) {
+                    push @elt, $self->get_migrate_doc( $elt_name, 'is computed with', $comp );
+                }
             }
         }
 
@@ -1448,15 +1596,16 @@ sub get_migrate_doc {
     my $mv    = $migr->{variables};
     my $mform = $migr->{formula};
 
-    if ( $migr->{use_eval} ) { $mform =~ s/^/ /mg; $mform = "\n\n$mform\n\n"; }
-    else                     { $mform = "'C<$mform>' " }
+    if ( $mform =~ /\n/) { $mform =~ s/^/ /mg; $mform = "\n\n$mform\n\n"; }
+    else                 { $mform = "'C<$mform>' " }
 
-    my $mdoc = "Note: $elt_name $desc ${mform}and with "
-        . join( ", ", map { qq!\$$_ => "C<$mv->{$_}>"! } sort keys %$mv );
+    my $mdoc = "Note: $elt_name $desc ${mform}and with: \n\n=over\n\n=item *\n\n"
+        . join( "\n\n=item *\n\n", map { qq!C<\$$_> => C<$mv->{$_}>! } sort keys %$mv );
     if ( my $rep = $migr->{replace} ) {
-        $mdoc .=
-            ' and ' . join( ", ", map { qq!'C<\$replace{$_}>' => "C<$rep->{$_}>"! } sort keys %$rep );
+        $mdoc .= "\n\n=item *\n\n"
+            . join( "\n\n=item *\n\n", map { qq!C<\$replace{$_}> => C<$rep->{$_}>! } sort keys %$rep );
     }
+    $mdoc .= "\n\n=back\n\n";
 
     return ( $mdoc, '' );
 }
@@ -1490,16 +1639,33 @@ sub get_element_description {
 
     $info .= "Type " . ( $vt || $type ) . $of . '. ';
 
-    foreach (qw/choice default upstream_default/) {
-        my $item = $elt_info->{$_};
+    foreach my $name (qw/choice/) {
+        my $item = $elt_info->{$name};
         next unless defined $item;
-        my @list = ref($item) ? @$item : ($item);
-        $info .= "$_: '" . join( "', '", @list ) . "'. ";
+        $info .= "$name: '" . join( "', '", @$item ) . "'. ";
+    }
+
+    my @default_info = ();
+    # assemble in over item for string value_type
+    foreach my $name (qw/default upstream_default/) {
+        my $item = $elt_info->{$name};
+        next unless defined $item;
+        push @default_info, [$name, $item] ;
     }
 
     my $elt_help = $self->get_element_value_help($elt_info);
 
-    return $desc . "I<< $info >> " . $elt_help;
+    # breaks pod if $info is multiline
+    my $ret = $desc . "I< $info > ";
+
+    if (@default_info) {
+        $ret .= "\n\n=over 4\n\n";
+        map { $ret .= "=item $_->[0] value :\n\n$_->[1]\n\n"; } @default_info;
+        $ret .= "=back\n\n";
+    }
+
+    $ret.= $elt_help;
+    return $ret;
 }
 
 sub get_element_value_help {
@@ -1517,34 +1683,28 @@ sub get_element_value_help {
 }
 
 sub generate_doc {
-    my ( $self, $top_class_name, $dir, $done ) = @_;
+    my ( $self, $top_class_name, $dir_str, $done ) = @_;
 
     $done //= {} ;
     my $res = $self->get_model_doc($top_class_name, $done);
 
-    if ( defined $dir and $dir ) {
+    if ( defined $dir_str and $dir_str ) {
         foreach my $class_name ( sort keys %$res ) {
-            my $file = $class_name;
-            $file =~ s!::!/!g;
-            my $pl_file  = $dir . "/$file.pl";
-            my $pod_file = $dir . "/$file.pod";
-            my $pod_dir  = $pod_file;
-            $pod_dir =~ s!/[^/]+$!!;
-            make_path( $pod_dir, { mode => 0755 } ) unless -d $pod_dir;
+            my $dir = path($dir_str);
+            $dir->mkpath() unless $dir->exists;
+            my $file_path = $class_name;
+            $file_path =~ s!::!/!g;
+            my $pl_file  = $dir->child("$file_path.pl");
+            $pl_file->parent->mkpath unless $pl_file->parent->exists;
+            my $pod_file = $dir->child("$file_path.pod");
 
-            # -M returns age of file, not date or epoch. hence greater is older
             my $old = '';
-            if ( -e $pod_file ) {
-                my $fh = IO::File->new( $pod_file, 'r' ) || die "Can't open $pod_file: $!";
-                $old = join( '', $fh->getlines );
-                $fh->close;
+            if ($pod_file->exists ) {
+                $old = $pod_file->slurp_utf8;
             }
             if ( $old ne $res->{$class_name} ) {
-                my $fh = IO::File->new( $pod_file, 'w' ) || die "Can't open $pod_file: $!";
-                $fh->binmode(":utf8");
-                $fh->print( $res->{$class_name} );
-                $fh->close;
-                print "Wrote documentation in $pod_file\n";
+                $pod_file->spew_utf8( $res->{$class_name} );
+                say "Wrote documentation in $pod_file";
             }
         }
     }
@@ -1686,7 +1846,7 @@ Config::Model - Create tools to validate, migrate and edit configuration files
 
 =head1 VERSION
 
-version 2.097
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -1713,9 +1873,9 @@ version 2.097
  $model ->create_config_class (
    name => "MiniModel",
    element => [ [qw/foo bar baz/ ] => { type => 'leaf', value_type => 'uniline' }, ],
-   read_config => { backend => 'IniFile', auto_create => 1,
-                    config_dir => '.', file => 'mini.ini',
-                  }
+   rw_config => { backend => 'IniFile', auto_create => 1,
+                  config_dir => '.', file => 'mini.ini',
+                }
  ) ;
 
  # create instance (Config::Model::Instance object)
@@ -1737,9 +1897,9 @@ version 2.097
  $ mkdir -p lib/Config/Model/models/
  $ echo "[ { name => 'MiniModel', \
              element => [ [qw/foo bar baz/ ] => { type => 'leaf', value_type => 'uniline' }, ], \
-             read_config => { backend => 'IniFile', auto_create => 1, \
-                              config_dir => '.', file => 'mini.ini', \
-                            } \
+             rw_config => { backend => 'IniFile', auto_create => 1, \
+                            config_dir => '.', file => 'mini.ini', \
+                          } \
            } \
          ] ; " > lib/Config/Model/models/MiniModel.pl
  # require App::Cme
@@ -1766,8 +1926,7 @@ A description of the structure and constraints of the project's configuration
 
 =item *
 
-A module to read and write configuration data. This can be one of the
-read/write backends provided by Config::Model or a custom backend.
+A module to read and write configuration data (aka a backend class).
 
 =back
 
@@ -2031,11 +2190,24 @@ L<Config::Model::TkUI>.
 
 =head1 Constructor
 
-Simply call new without parameters:
-
  my $model = Config::Model -> new ;
 
-This creates an object to host your model.
+creates an object to host your model.
+
+=head2 Constructor parameters
+
+=over
+
+=item log_level
+
+Specify minimal log level. Default is C<WARN>. Can be C<INFO>,
+C<DEBUG> or C<TRACE> to get more logs. Can also be C<ERROR> to get
+less traces.
+
+This parameter is used to override the log level specified in log
+configuration file.
+
+=back
 
 =head1 Configuration Model
 
@@ -2202,12 +2374,12 @@ Example:
    }
  ];
 
-=head1 Configuration instance
+=head1 Configuration instance methods
 
 A configuration instance is created from a model and is the starting
 point of a configuration tree.
 
-=head2 instance(...)
+=head2 instance
 
 An instance must be created with a model name (using the root class
 name) or an application name (as shown by "L<cme> C<list>" command).
@@ -2256,10 +2428,22 @@ been created:
 
  my $inst = $model->instance( name => 'test1' );
 
-=head2 cme ( ... )
+=head2 get_instance
+
+Retrieve an existing instance using its name.
+
+ my $inst = $model->get_instance('test1' );
+
+=head2 has_instance
+
+Check if an instance name already exists
+
+  my $maybe = $model->has_instance('test1');
+
+=head2 cme
 
 This method is syntactic sugar for short program. It creates a new
-C<Config::Model> object and returns a new instance. See L</instance(...)>
+C<Config::Model> object and returns a new instance. See L</instance>
 for the parameters.
 
 =head1 Configuration class
@@ -2698,7 +2882,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
@@ -2751,7 +2935,7 @@ L<http://cpants.cpanauthors.org/dist/Config-Model>
 
 CPAN Testers
 
-The CPAN Testers is a network of smokers who run automated tests on uploaded CPAN distributions.
+The CPAN Testers is a network of smoke testers who run automated tests on uploaded CPAN distributions.
 
 L<http://www.cpantesters.org/distro/C/Config-Model>
 
