@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::AnyId;
-$Config::Model::AnyId::VERSION = '2.097';
+$Config::Model::AnyId::VERSION = '2.117';
 use 5.010;
 
 use Mouse;
@@ -134,7 +134,7 @@ around BUILDARGS => sub {
 
 has [qw/backup cargo/] => ( is => 'ro', isa => 'HashRef', required => 1 );
 has warp => ( is => 'ro', isa => 'Maybe[HashRef]' );
-has [qw/morph/] => ( is => 'ro', isa => 'Bool' );
+has [qw/morph/] => ( is => 'ro', isa => 'Bool', default => 0 );
 has content_warning_list => ( is => 'rw', isa => 'ArrayRef', default => sub { []; } );
 has [qw/cargo_class max_index index_class index_type/] =>
     ( is => 'rw', isa => 'Maybe[Str]' );
@@ -198,7 +198,7 @@ sub set_properties {
     # these are handled by Node or Warper
     map { delete $args{$_} } qw/level/;
 
-    $logger->debug( $self->name, " set_properties called with @_" );
+    $logger->trace( $self->name, " set_properties called with @_" );
 
     map { $self->{$_} = delete $args{$_} if defined $args{$_} } @common_params;
 
@@ -277,18 +277,35 @@ sub set_properties {
 
 sub create_default_with_init {
     my $self = shift;
+    my $idx = shift;
 
     return unless defined $self->{default_with_init};
 
     my $h = $self->{default_with_init};
     foreach my $def_key ( keys %$h ) {
-        my $v_obj = $self->fetch_with_id($def_key);
-        if ( $v_obj->get_type eq 'leaf' ) {
-            $v_obj->store( $h->{$def_key} );
-        }
-        else {
-            $v_obj->load( $h->{$def_key} );
-        }
+        $self->create_default_content($def_key);
+    }
+}
+
+sub create_default_content {
+    my $self = shift;
+    my $idx = shift //  die "missing index";
+
+    return unless defined $self->{default_with_init};
+
+    my $def = $self->{default_with_init}{$idx};
+    return unless defined $def; # no default content to create for $idx
+
+    return if $self->_defined($idx) ; # object already created
+
+    $self->auto_vivify($idx);
+
+    my $v_obj = $self->_fetch_with_id($idx);
+    if ( $v_obj->get_type eq 'leaf' ) {
+        $v_obj->store( $def );
+    }
+    else {
+        $v_obj->load( $def );
     }
 }
 
@@ -401,7 +418,7 @@ sub handle_args {
 
 sub apply_fixes {
     my $self = shift;
-    $fix_logger->debug( $self->location . ": apply_fixes called" );
+    $fix_logger->trace( $self->location . ": apply_fixes called" );
 
     $self->deep_check( fix => 1, logger => $fix_logger );
 
@@ -422,9 +439,9 @@ sub notify_change {
     my $self = shift;
     my %args = @_;
 
-    $change_logger->debug( "called for ", $self->name, " from ", join( ' ', caller ),
+    $change_logger->trace( "called for ", $self->name, " from ", join( ' ', caller ),
         " with ", join( ' ', %args ) )
-        if $change_logger->is_debug;
+        if $change_logger->is_trace;
 
     # $idx may be undef if $self has changed, not necessarily its content
     my $idx = $args{index};
@@ -464,7 +481,7 @@ sub deep_check {
     my $self = shift;
     my @args = @_;
 
-    $deep_check_logger->debug("called on ".$self->name);
+    $deep_check_logger->trace("called on ".$self->name);
 
     map { $self->check_idx(@args, index => $_); } $self->fetch_all_indexes();
 
@@ -482,7 +499,7 @@ sub check_content {
 
     if ( $self-> needs_content_check ) {
 
-        $local_logger->debug( "Running check_content on ",$self->location );
+        $local_logger->trace( "Running check_content on ",$self->location );
         # need to keep track to update GUI
         $self-> flush_fixes;    # reset before check
 
@@ -645,7 +662,7 @@ sub check_duplicates {
     my $dup = $self->{duplicates};
     return if $dup eq 'allow';
 
-    $logger->debug("check_duplicates called");
+    $logger->trace("check_duplicates called");
     my %h;
     my @issues;
     my @to_delete;
@@ -691,7 +708,7 @@ sub fetch_with_id {
     my $check = $self->_check_check( $args{check} );
     my $idx   = $args{index};
 
-    $logger->debug( $self->name, " called for idx $idx" ) if $logger->is_debug;
+    $logger->trace( $self->name, " called for idx $idx" ) if $logger->is_trace;
 
     $idx = $self->{convert_sub}($idx)
         if ( defined $self->{convert_sub} and defined $idx );
@@ -707,6 +724,9 @@ sub fetch_with_id {
         or $check eq 'no';
 
     if ( $ok or $check eq 'no' ) {
+        # create another method
+        $self->create_default_content($idx); # no-op if idx exists
+
         $self->auto_vivify($idx) unless $self->_defined($idx);
         return $self->_fetch_with_id($idx);
     }
@@ -743,7 +763,7 @@ sub get {
 
     return unless ( $self->exists($item) or $autoadd );
 
-    $logger->debug("get: path $path, item $item");
+    $logger->trace("get: path $path, item $item");
 
     my $obj = $self->fetch_with_id( index => $item, %args );
     return $obj if ( ( $get_obj or $obj->get_type ne 'leaf' ) and not defined $new_path );
@@ -790,6 +810,11 @@ sub fetch_all {
     return map { $self->fetch_with_id($_); } @keys;
 }
 
+sub fetch {
+    my $self = shift;
+    return join(',', $self->fetch_all_values(@_) );
+}
+
 sub fetch_all_values {
     my $self  = shift;
     my %args  = @_ > 1 ? @_ : ( mode => shift );
@@ -938,7 +963,8 @@ sub delete {
 
     delete $self->{warning_hash}{$idx};
     my $ret = $self->_delete($idx);
-    $self->notify_change( note => "deleted entry $idx" );
+    # notification is not needed if the value was already delete or missing
+    $self->notify_change( note => "deleted entry $idx" ) if defined $ret;
     return $ret;
 }
 
@@ -1011,7 +1037,7 @@ Config::Model::AnyId - Base class for hash or list element
 
 =head1 VERSION
 
-version 2.097
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -1131,6 +1157,18 @@ to apply a fix), C<forbid>. Note that duplicates I<check cannot be
 performed when the duplicated value is stored>: this happens outside of
 this object. Duplicates can be check only after when the value is read.
 
+=item write_empty_value
+
+By default, hash entries without data are not saved in configuration
+files. Without data means the cargo of the hash key is empty: either
+its value is undef or all the values of the contained node are also empty.
+
+Set this parameter to 1 if the key must be saved in the configuration
+file even if the hash contains no value for that key.
+
+Note that writing hash entries without value may not be supported by
+all backends. Use with care. Supported only for hash elements.
+
 =item cargo
 
 Hash ref specifying the cargo held by the hash of list. This has must
@@ -1182,13 +1220,17 @@ or C<< default_keys => ['foo', 'bar'] >>.
 
 To perform special set-up on children nodes you can also use 
 
-   default_with_init =>  { 'foo' => 'X=Av Y=Bv' ,
-                           'bar' => 'Y=Av Z=Cv' }
+   default_with_init =>  {
+      foo => 'X=Av Y=Bv',
+      bar => 'Y=Av Z=Cv'
+   }
 
 When the hash contains leaves, you can also use:
 
-   default_with_init => { 'def_1' => 'def_1 stuff' ,
-                          'def_2' => 'def_2 stuff' }
+   default_with_init => {
+       def_1 => 'def_1 stuff',
+       def_2 => 'def_2 stuff'
+   }
 
 =item migrate_keys_from
 
@@ -1524,6 +1566,11 @@ The default value (defined by the configuration model)
 
 =back
 
+=head2 fetch
+
+Similar to L</fetch_all_values>, with the same parameters, Returns the
+result as a string with comma separated list values.
+
 =head2 fetch_all_indexes()
 
 Returns an array containing all indexes of the hash or list. Hash keys
@@ -1595,7 +1642,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
