@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Backend::Any;
-$Config::Model::Backend::Any::VERSION = '2.097';
+$Config::Model::Backend::Any::VERSION = '2.117';
 use Carp;
 use strict;
 use warnings;
@@ -27,14 +27,12 @@ has 'node' => (
     isa      => 'Config::Model::Node',
     weak_ref => 1,
     required => 1,
-    handles => ['show_message', 'instance'],
+    handles => [ qw/show_message instance get_element_names/],
 );
 
+sub skip_open { return 0; }
+
 sub suffix {
-    my $self = shift;
-    $logger->info(
-        "Internal warning: suffix called for backend $self->{name}.This method can be overloaded"
-    );
     return undef;
 }
 
@@ -114,7 +112,7 @@ sub associates_comments_with_data {
         if ($data) {
             my $note = '';
             $note = join( "\n", @comments ) if @comments;
-            $logger->debug("associates_comments_with_data: '$note' with '$data'");
+            $logger->trace("associates_comments_with_data: '$note' with '$data'");
             push @result, [ $data, $note ];
             @comments = ();
         }
@@ -185,7 +183,7 @@ Config::Model::Backend::Any - Virtual class for other backends
 
 =head1 VERSION
 
-version 2.097
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -194,11 +192,6 @@ version 2.097
 
  extends 'Config::Model::Backend::Any';
 
- # optional
- sub suffix {
-   return '.conf';
- }
-
  # mandatory
  sub read {
     my $self = shift ;
@@ -277,23 +270,23 @@ L<Config::Model::Node> specification.
 Let's say your new backend is C<Config::Model::Backend::Foo>. This new backend
 can be specified with:
 
- read_config  => [ {
+ rw_config  => {
     backend    => 'Foo' , # can also be 'foo'
     config_dir => '/etc/cfg_dir'
     file       => 'foo.conf', # optional
- }]
+ }
 
 (The backend class name is constructed with C<ucfirst($backend_name)>)
 
-C<read_config> can also have custom parameters that are passed
+C<rw_config> can also have custom parameters that are passed
 verbatim to C<Config::Model::Backend::Foo> methods:
 
- read_config  => [ {
+ rw_config  => {
     backend    => 'Foo' , # can also be 'foo'
     config_dir => '/etc/cfg_dir'
     file       => 'foo.conf', # optional
     my_param   => 'my_value',
- } ]
+ }
 
 C<Config::Model::Backend::Foo> class must inherit (extend)
 L<Config::Model::Backend::Any> and is expected to provide the
@@ -359,8 +352,8 @@ the C<write> call-back with the file handle with C<io_handle>
 parameter. C<write> should use this handle to write data in the target
 configuration file.
 
-If this behavior causes problem, the solution is either to override
-C<skip_open> method in your backend that returns C<1>.
+If this behavior causes problem, the solution is to override
+C<skip_open> method in your backend to return C<1>.
 
 =back
 
@@ -392,10 +385,6 @@ Whether the backend supports reading and writing annotation (a.k.a
 comments). Default is 0. Override this method to return 1 if your
 backend supports annotations.
 
-=head2 suffix
-
-Suffix of the configuration file. This method returns C<undef> by default.
-
 =head2 read
 
 Read the configuration file. This method must be overridden.
@@ -490,6 +479,30 @@ Write data and comments in the C<io_handle> (if defined). Comments are written b
 Returns the string written to the io_handle. If a data is undef, the comment is written on its own
 line.
 
+=head1 Replacing a custom backend
+
+Custom backend are now deprecated and must be replaced with a class inheriting this module.
+
+Please:
+
+=over
+
+=item *
+
+Rename your class to begin with C<Config::Model::Backend::>
+
+=item *
+
+Add C<use Mouse ;> and C<extends 'Config::Model::Backend::Any';> in the header of your custom class.
+
+=item *
+
+Add C<my $self = shift;> as the beginning of C<read> and C<write> functions... well... methods.
+
+=back
+
+Here's an L<example of such a change|https://github.com/dod38fr/config-model/commit/c3b7007ad386cb2356c5ac1499fe51bdf492b19a>.
+
 =head1 AUTHOR
 
 Dominique Dumont, (ddumont at cpan dot org)
@@ -507,7 +520,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
