@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Backend::Fstab;
-$Config::Model::Backend::Fstab::VERSION = '2.097';
+$Config::Model::Backend::Fstab::VERSION = '2.117';
 use Mouse;
 use Carp;
 use Log::Log4perl qw(get_logger :levels);
@@ -17,8 +17,6 @@ extends 'Config::Model::Backend::Any';
 
 my $logger = get_logger("Backend::Fstab");
 
-sub suffix { return ''; }
-
 sub annotation { return 1; }
 
 my %opt_r_translate = (
@@ -65,7 +63,7 @@ sub read {
         my $fs_obj = $self->node->fetch_element('fs')->fetch_with_id($label);
 
         if ($comment) {
-            $logger->debug("Annotation: $comment\n");
+            $logger->trace("Annotation: $comment\n");
             $fs_obj->annotation($comment);
         }
 
@@ -164,7 +162,7 @@ Config::Model::Backend::Fstab - Read and write config from fstab file
 
 =head1 VERSION
 
-version 2.097
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -227,7 +225,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
