@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Backend::IniFile;
-$Config::Model::Backend::IniFile::VERSION = '2.097';
+$Config::Model::Backend::IniFile::VERSION = '2.117';
 use Carp;
 use Mouse;
 use 5.10.0;
@@ -20,8 +20,6 @@ use base qw/Config::Model::Backend::Any/;
 
 my $logger = get_logger("Backend::IniFile");
 
-sub suffix { return '.ini'; }
-
 sub annotation { return 1; }
 
 sub read {
@@ -46,6 +44,7 @@ sub read {
     my $section_map = $args{section_map}         || {};
     my $split_reg   = $args{split_list_value};
     my $check       = $args{check}               || 'yes';
+    my $assign_char = $args{assign_char}         || '=';
     my $obj         = $self->node;
 
     my %force_lc;
@@ -88,11 +87,13 @@ sub read {
             $section = $force_lc{section} ? lc($1) : $1;
             my $remap = $section_map->{$section} || '';
             if ( $remap eq '!' ) {
+                # section_map maps section to root node
                 $section_ref = $ini_data;
                 $comment_path = $section_path = '';
                 $logger->debug("step 1: found node <top> [$section]");
             }
             elsif ($remap) {
+                # section_map maps section to some node
                 $section_ref = {};
                 $logger->debug("step 1: found node $remap [$section]");
                 $section_path = $comment_path =
@@ -117,7 +118,7 @@ sub read {
             }
         }
         else {
-            my ( $name, $val ) = split( /\s*=\s*/, $vdata, 2 );
+            my ( $name, $val ) = split( /\s*$assign_char\s*/, $vdata, 2 );
             $name = lc($name) if $force_lc{key};
             $val  = lc($val)  if $force_lc{value};
             $comment_path = $section_path . ' ' . $self->set_or_push( $section_ref, $name, $val );
@@ -206,6 +207,7 @@ sub _write_list{
 
     my $join_list = $args->{join_list_value};
     my $delimiter = $args->{comment_delimiter} || '#';
+    my $assign_with = $args->{assign_with} // $args->{assign_char} // ' = ';
     my $obj = $node->fetch_element($elt);
 
     my $obj_note = $obj->annotation;
@@ -215,7 +217,7 @@ sub _write_list{
         my $v = join( $join_list, @v );
         if ( length($v) ) {
             $logger->debug("writing joined list elt $elt -> $v");
-            $res .= $self->write_data_and_comments( undef, $delimiter, "$elt=$v", $obj_note );
+            $res .= $self->write_data_and_comments( undef, $delimiter, "$elt$assign_with$v", $obj_note );
         }
     }
     else {
@@ -225,11 +227,11 @@ sub _write_list{
             if ( length $v ) {
                 $logger->debug("writing list elt $elt -> $v");
                 $res .=
-                    $self->write_data_and_comments( undef, $delimiter, "$elt=$v",
+                    $self->write_data_and_comments( undef, $delimiter, "$elt$assign_with$v",
                                                     $obj_note . $note );
             }
             else {
-                $logger->debug("NOT writing undef or empty list elt");
+                $logger->trace("NOT writing undef or empty list elt");
             }
         }
     }
@@ -242,6 +244,7 @@ sub _write_check_list{
 
     my $join_check_list = $args->{join_check_list_value};
     my $delimiter = $args->{comment_delimiter} || '#';
+    my $assign_with = $args->{assign_with} // $args->{assign_char} // ' = ';
     my $obj = $node->fetch_element($elt);
 
     my $obj_note = $obj->annotation;
@@ -250,13 +253,13 @@ sub _write_check_list{
         my $v = join( $join_check_list, $obj->get_checked_list() );
         if ( length($v) ) {
             $logger->debug("writing check_list elt $elt -> $v");
-            $res .= $self->write_data_and_comments( undef, $delimiter, "$elt=$v", $obj_note );
+            $res .= $self->write_data_and_comments( undef, $delimiter, "$elt$assign_with$v", $obj_note );
         }
     }
     else {
         foreach my $v ( $obj->get_checked_list() ) {
             $logger->debug("writing joined check_list elt $elt -> $v");
-            $res .= $self->write_data_and_comments( undef, $delimiter, "$elt=$v", $obj_note );
+            $res .= $self->write_data_and_comments( undef, $delimiter, "$elt$assign_with$v", $obj_note );
         }
     }
     return $res;
@@ -268,6 +271,7 @@ sub _write_leaf{
 
     my $write_bool_as = $args->{write_boolean_as};
     my $delimiter = $args->{comment_delimiter} || '#';
+    my $assign_with = $args->{assign_with} // $args->{assign_char} // ' = ';
     my $obj = $node->fetch_element($elt);
 
     my $obj_note = $obj->annotation;
@@ -278,10 +282,10 @@ sub _write_leaf{
     }
     if ( defined $v and length $v ) {
         $logger->debug("writing leaf elt $elt -> $v");
-        $res .= $self->write_data_and_comments( undef, $delimiter, "$elt=$v", $obj_note );
+        $res .= $self->write_data_and_comments( undef, $delimiter, "$elt$assign_with$v", $obj_note );
     }
     else {
-        $logger->debug("NOT writing undef or empty leaf elt");
+        $logger->trace("NOT writing undef or empty leaf elt");
     }
     return $res;
 }
@@ -341,14 +345,14 @@ sub _write {
     my $node          = $args{object};
     my $delimiter     = $args{comment_delimiter} || '#';
 
-    $logger->debug( "called on ", $node->name );
+    $logger->trace( "called on ", $node->name );
     my $res = '';
 
     # Using Config::Model::ObjTreeScanner would be overkill
     # first write list and element, then classes
     foreach my $elt ( $node->get_element_name ) {
         my $type = $node->element_type($elt);
-        $logger->debug("first loop on elt $elt type $type");
+        $logger->trace("first loop on elt $elt type $type");
         next if $type =~ /node/ or $type eq 'hash';
 
         if ( $type eq 'list' ) {
@@ -370,7 +374,7 @@ sub _write {
 
     foreach my $elt ( $node->get_element_name ) {
         my $type = $node->element_type($elt);
-        $logger->debug("second loop on elt $elt type $type");
+        $logger->trace("second loop on elt $elt type $type");
         next unless $type =~ /node/ or $type eq 'hash';
         my $obj = $node->fetch_element($elt);
 
@@ -384,7 +388,7 @@ sub _write {
         }
     }
 
-    $logger->debug( "done on ", $node->name );
+    $logger->trace( "done on ", $node->name );
 
     return $res;
 }
@@ -408,7 +412,7 @@ Config::Model::Backend::IniFile - Read and write config as a INI file
 
 =head1 VERSION
 
-version 2.097
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -440,15 +444,13 @@ version 2.097
         },
     ],
 
-   read_config  => [
-        {
-            backend => 'IniFile',
-            config_dir => '/tmp',
-            file  => 'foo.conf',
-            store_class_in_hash => 'ini_class',
-            auto_create => 1,
-        }
-    ],
+   rw_config  => {
+     backend => 'IniFile',
+     config_dir => '/tmp',
+     file  => 'foo.conf',
+     store_class_in_hash => 'ini_class',
+     auto_create => 1,
+   }
  );
 
  my $inst = $model->instance(root_class_name => 'MyClass' );
@@ -583,6 +585,14 @@ Array ref. Reserved for boolean value. Specify how to write a boolean value.
 Default is C<[0,1]> which may not be the most readable. C<write_boolean_as> can be 
 specified as C<['false','true']> or C<['no','yes']>. 
 
+=item assign_char
+
+Character used to assign value in INI file. Default is C<=>.
+
+=item assign_with
+
+String used write assignment in INI file. Default is "C< = >".
+
 =back
 
 =head1 Mapping between INI structure and model
@@ -634,45 +644,64 @@ The model has this structure:
     \- key B (value is node of class_A)
        \- element-bar
 
-In this case, the C<my_class_holder> name is specified in C<read_config> with C<store_class_in_hash> 
+In this case, the C<my_class_holder> name is specified in C<rw_config> with C<store_class_in_hash> 
 parameter:
 
-    read_config  => [
-        { 
-            backend => 'IniFile',
-            config_dir => '/tmp',
-            file  => 'foo.ini',
-            store_class_in_hash => 'my_class_holder',
-        }
-    ],
+    rw_config  => {
+      backend => 'IniFile',
+      config_dir => '/tmp',
+      file  => 'foo.ini',
+      store_class_in_hash => 'my_class_holder',
+    }
 
 Of course they are exceptions. For instance, in C<Multistrap>, the C<[General]> 
 INI class must be mapped to a specific node object. This can be specified
 with the C<section_map> parameter: 
 
-    read_config  => [
-        { 
-            backend => 'IniFile',
-            config_dir => '/tmp',
-            file  => 'foo.ini',
-            store_class_in_hash => 'my_class_holder',
-            section_map => { 
-                General => 'general_node',
-            }
-        }
-    ],
+    rw_config  => }
+       backend => 'IniFile',
+       config_dir => '/tmp',
+       file  => 'foo.ini',
+       store_class_in_hash => 'my_class_holder',
+       section_map => {
+           General => 'general_node',
+       }
+    }
 
 C<section_map> can also map an INI class to the root node:
 
-    read_config => [
-        {
-            backend => 'ini_file',
-            store_class_in_hash => 'sections',
-            section_map => {
-                General => '!'
-            },
-        }
-    ],
+    rw_config => {
+      backend => 'ini_file',
+      store_class_in_hash => 'sections',
+      section_map => {
+          General => '!'
+      },
+    }
+
+=head1 Handle key value files
+
+This backend is able to handle simple configuration files where the
+values are written as key value pairs like:
+
+ foo = bar
+
+or
+
+ foo: bar
+
+The option C<assign_char> is used to specify which character is used
+to assign a value in the file (white spaces are ignored).
+C<assign_char> is "C<=>" (the default) in the first example, and
+"C<:>" in the second.
+
+The C<assign_with> is used to control how the file is written back. E.g:
+
+ foo=bar   # the default
+ foo= bar  # assign_with is "= "
+ foo = bar # assign_with is " = "
+ foo:bar   # assign_char is ':', assign_with is the default
+ foo: bar  # assign_char is ':', assign_with is ": "
+ foo : bar # assign_char is ':', assign_with is " : "
 
 =head1 Methods
 
@@ -711,7 +740,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
