@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Backend::Json;
-$Config::Model::Backend::Json::VERSION = '2.097';
+$Config::Model::Backend::Json::VERSION = '2.117';
 use Carp;
 use strict;
 use warnings;
@@ -21,8 +21,6 @@ use JSON;
 
 my $logger = get_logger("Backend::Json");
 
-sub suffix { return '.json'; }
-
 sub read {
     my $self = shift;
     my %args = @_;
@@ -93,7 +91,7 @@ Config::Model::Backend::Json - Read and write config as a JSON data structure
 
 =head1 VERSION
 
-version 2.097
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -118,13 +116,12 @@ version 2.097
             },
         },
     ],
-  read_config  => [
-                    { backend => 'Json' ,
-                      config_dir => '/tmp',
-                      file  => 'foo.json',
-                      auto_create => 1,
-                    }
-                  ],
+  rw_config  => {
+    backend => 'Json' ,
+    config_dir => '/tmp',
+    file  => 'foo.json',
+    auto_create => 1,
+  }
  ) ;
 
  my $inst = $model->instance(root_class_name => 'MyClass' );
@@ -199,7 +196,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
