@@ -1,28 +1,36 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Backend::PlainFile;
-$Config::Model::Backend::PlainFile::VERSION = '2.088';
+$Config::Model::Backend::PlainFile::VERSION = '2.117';
+use 5.10.1;
 use Carp;
 use Mouse;
 use Config::Model::Exception;
-use File::Path;
+use Path::Tiny;
 use Log::Log4perl qw(get_logger :levels);
 
 extends 'Config::Model::Backend::Any';
 
-my $logger = get_logger("Backend::PlainFile");
+with "Config::Model::Role::ComputeFunction";
+with "Config::Model::Role::FileHandler";
 
-sub suffix { return ''; }
+my $logger = get_logger("Backend::PlainFile");
 
 sub annotation { return 0; }
 
+# remember that a read backend (and its config file(s)) is attached to a node
+# OTOH, PlainFile backend deal with files that are attached to elements of a node.
+# Hence the files must not be managed by backend manager.
+
+# file not opened by BackendMgr
+# file_path, io_handle are undef
 sub skip_open { 1; }
 
 sub read {
@@ -39,16 +47,20 @@ sub read {
     # check      => yes|no|skip
 
     my $check = $args{check} || 'yes';
-    my $dir   = $args{config_dir};
     my $node  = $args{object};
-    $logger->debug( "called on node", $node->name );
+    $logger->trace( "called on node ", $node->name );
 
     # read data from leaf element from the node
-    foreach my $elt ( $node->get_element_name() ) {
-        my $file = $args{root} . $dir . $elt;
-        $logger->trace("looking for plainfile $file");
-
+    # do not trigger warp when getting element names
+    foreach my $elt ( $node->get_element_names(all => 1) ) {
         my $obj = $args{object}->fetch_element( name => $elt );
+
+        my $file_name = $args{file} ? $obj->compute_string($args{file}) : $elt;
+        my $dir = $self->get_tuned_config_dir(%args);
+        my $file = $dir->child($file_name);
+
+        $logger->trace("looking for plainfile $file ");
+
         my $type = $obj->get_type;
 
         if ( $type eq 'leaf' ) {
@@ -61,7 +73,7 @@ sub read {
             $self->read_hash( $obj, $elt, $check, $file, \%args );
         }
         else {
-            $logger->debug("PlainFile read skiped $type $elt");
+            $logger->debug("PlainFile read skipped $type $elt");
         }
 
     }
@@ -69,51 +81,31 @@ sub read {
     return 1;
 }
 
-#
-# New subroutine "open_for_read" extracted - Thu Jul 21 13:36:52 2011.
-#
-sub open_for_read {
-    my ( $self, $file, $elt ) = @_;
-
-    return unless -e $file;
-
-    my $fh = new IO::File;
-    $fh->open($file) or die "Cannot open $file:$!";
-    $fh->binmode(":utf8");
-    $logger->trace("found file $file for element $elt");
-
-    return ($fh);
-}
-
-#
-# New subroutine "read_leaf" extracted - Thu Jul 21 12:58:06 2011.
-#
 sub read_leaf {
     my ( $self, $obj, $elt, $check, $file, $args ) = @_;
 
-    my $fh = $self->open_for_read( $file, $elt ) or return;
+    return unless $file->exists;
 
-    my $v = join( '', $fh->getlines );
-    chomp $v unless $obj->value_type eq 'string';
+    my $v = $file->slurp_utf8;
+    chomp($v) unless $obj->value_type eq 'string';
+    if ($logger->is_trace) {
+        (my $str = $v) =~ s/\n.*/[...]/s;
+        $logger->trace("storing leaf value '$str' from $file ");
+    }
     $obj->store( value => $v, check => $check );
 }
 
-#
-# New subroutine "read_list" extracted - Thu Jul 21 12:58:36 2011.
-#
 sub read_list {
     my ( $self, $obj, $elt, $check, $file, $args ) = @_;
 
-    my $fh = $self->open_for_read( $file, $elt ) or return;
+    return unless $file->exists;
+
+    my @v = $file->lines_utf8({ chomp => 1});
+    $logger->trace("storing list value @v from $file ");
 
-    my @v = $fh->getlines;
-    chomp @v;
     $obj->store_set(@v);
 }
 
-#
-# New subroutine "read_hash" extracted - Thu Jul 21 12:58:50 2011.
-#
 sub read_hash {
     my ( $self, $obj, $elt, $check, $file, $args ) = @_;
     $logger->debug("PlainFile read skipped hash $elt");
@@ -133,43 +125,77 @@ sub write {
     # check      => yes|no|skip
 
     my $check = $args{check} || 'yes';
-    my $dir = $args{root} . $args{config_dir};
-    mkpath( $dir, { mode => 0755 } ) unless -d $dir;
+    my $cfg_dir = $args{config_dir};
+    my $dir = $self->get_tuned_config_dir(%args);
+    $dir->mkpath({ mode => 0755 } ) unless $dir->is_dir;
+
     my $node = $args{object};
     $logger->debug( "PlainFile write called on node ", $node->name );
 
     # write data from leaf element from the node
     foreach my $elt ( $node->get_element_name() ) {
-        my $file = $dir . $elt;
-
         my $obj = $args{object}->fetch_element( name => $elt );
+
+        my $file_name = $args{file} ? $obj->compute_string($args{file}) : $elt;
+        my $file = $dir->child($file_name);
+
         my $type = $obj->get_type;
         my @v;
 
         if ( $type eq 'leaf' ) {
-            $v[0] = $obj->fetch( check => $args{check} );
-            $v[0] .= "\n" unless $obj->value_type eq 'string';
+            my $v = $obj->fetch( check => $args{check} );
+            $v .= "\n" if defined $v and $obj->value_type ne 'string';
+            push @v, $v if defined $v;
         }
         elsif ( $type eq 'list' ) {
             @v = map { "$_\n" } $obj->fetch_all_values;
         }
         else {
             $logger->debug("PlainFile write skipped $type $elt");
+            next;
         }
 
         if (@v) {
-            $logger->trace("PlainFile write opening $file to write");
-            my $fh = new IO::File;
-            $fh->open( $file, '>' ) or die "Cannot open $file:$!";
-            $fh->binmode(":utf8");
-            $fh->print(@v);
-            $fh->close;
+            $logger->trace("PlainFile write opening $file to write $elt");
+            $file->spew_utf8(@v);
+            $file->chmod($args{file_mode}) if $args{file_mode};
+        }
+        elsif ($file->exists) {
+            $logger->trace("PlainFile delete $file");
+            $file->remove;
         }
     }
 
     return 1;
 }
 
+sub delete {
+    my $self = shift;
+    my %args = @_;
+
+    # args are:
+    # object     => $obj,         # Config::Model::Node object
+    # root       => './my_test',  # fake root directory, userd for tests
+    # config_dir => /etc/foo',    # absolute path read
+    # file       => 'foo.conf',   # file name
+    # file_path  => './my_test/etc/foo/foo.conf'
+    # io_handle  => $io           # IO::File object
+    # check      => yes|no|skip
+
+    my $dir = $self->get_tuned_config_dir(%args);
+    my $node = $args{object};
+    $logger->debug( "PlainFile delete called on deleted node");
+
+    # write data from leaf element from the node
+    foreach my $elt ( $node->get_element_name() ) {
+        my $obj = $node->fetch_element( name => $elt );
+
+        my $file = $dir->child($args{file} ? $obj->compute_string($args{file}) : $elt);
+        $logger->info( "Removing $file (deleted node)" );
+        $file->remove;
+    }
+}
+
 no Mouse;
 __PACKAGE__->meta->make_immutable;
 
@@ -189,7 +215,7 @@ Config::Model::Backend::PlainFile - Read and write config as plain file
 
 =head1 VERSION
 
-version 2.088
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -202,12 +228,10 @@ version 2.088
     element => [ 
         [qw/source new/] => { qw/type leaf value_type uniline/ },
     ],
-    read_config  => [ 
-        { 
-            backend => 'plain_file', 
-            config_dir => '/tmp',
-        },
-    ],
+    rw_config  => {
+      backend => 'plain_file',
+      config_dir => '/tmp',
+    },
  );
  
  my $inst = $model->instance(root_class_name => 'WithPlainFile' );
@@ -223,11 +247,61 @@ with C<foo> and C<yes> inside.
 =head1 DESCRIPTION
 
 This module is used directly by L<Config::Model> to read or write the
-content of a configuration tree written in several files. 
+content of a configuration tree written in several files.
 Each element of the node is written in a plain file.
 
-This module supports currently only leaf and list elements.  
-In the case of C<list> element, each line of the file is a value of the list.
+=head1 Element type and file mapping
+
+Element values are written in one or several files depending on their type.
+
+=over
+
+=item leaf
+
+The leaf value is written in one file. This file can have several lines if the leaf
+type is C<string>
+
+=item list
+
+The list content is written in one file. Each line of the file is a
+value of the list.
+
+=item hash
+
+Not supported
+
+=back
+
+=head1 File mapping
+
+By default, the configuration file is named after the element name
+(like in synopsis above).
+
+The C<file> parameter can also be used to specify a file name that
+take into account the path in the tree using C<&index()> and
+C<&element()> functions from L<Config::Model::Role::ComputeFunction>.
+
+For instance, with the following model:
+
+    class_name => "Foo",
+    element => [
+        string_a => { type => 'leaf', value_type => 'string'}
+        string_b => { type => 'leaf', value_type => 'string'}
+    ],
+    rw_config => {
+        backend => 'PlainFile',
+        config_dir => 'foo',
+        file => '&element(-).&element',
+        file_mode => 0644,  # optional
+    }
+
+If the configuration is loaded with C<example string_a=something
+string_b=else>, this backend writes "C<something>" in file
+C<example.string_a> and C<else> in file C<example.string_b>.
+
+C<file_mode> parameter can be used to set the mode of the written
+file. C<file_mode> value can be in any form supported by
+L<Path::Tiny/chmod>.
 
 =head1 Methods
 
@@ -265,7 +339,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
