@@ -1,15 +1,15 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-
 package Config::Model::Backend::Yaml;
-$Config::Model::Backend::Yaml::VERSION = '2.097';
+$Config::Model::Backend::Yaml::VERSION = '2.117';
+use 5.10.1;
 use Carp;
 use strict;
 use warnings;
@@ -18,11 +18,16 @@ use File::Path;
 use Log::Log4perl qw(get_logger :levels);
 
 use base qw/Config::Model::Backend::Any/;
-use YAML::Any;
 
 my $logger = get_logger("Backend::Yaml");
 
-sub suffix { return '.yml'; }
+sub load_yaml_parser {
+    my $self = shift;
+    my $file = my $class = shift // 'YAML::Tiny';
+    $file =~ s!::!/!g;
+    require "$file.pm" || die "cannot load YAML parser $class";
+    import $class;
+}
 
 sub single_element {
     my $self = shift;
@@ -50,13 +55,22 @@ sub read {
 
     return 0 unless defined $args{io_handle};    # no file to read
 
+    # load YAML parser and import Load and Dump
+    $self->load_yaml_parser($args{yaml_class});
+
     # load yaml file
     my $yaml = join( '', $args{io_handle}->getlines );
 
     # convert to perl data
-    my $perl_data = Load $yaml ;
+    my $perl_data = Load($yaml) ;
     if ( not defined $perl_data ) {
-        $logger->warn("No data found in YAML file $args{file_path}");
+        my $msg = "No data found in YAML file $args{file_path}";
+        if ($args{auto_create}) {
+            $logger->info($msg);
+        }
+        else {
+            $logger->warn($msg);
+        }
         return 1;
     }
 
@@ -80,6 +94,9 @@ sub write {
     # io_handle  => $io           # IO::File object
     # check      => yes|no|skip
 
+    # load YAML parser and import Load and Dump
+    $self->load_yaml_parser($args{yaml_class});
+
     croak "Undefined file handle to write"
         unless defined $args{io_handle};
 
@@ -87,7 +104,7 @@ sub write {
 
     my $perl_data = $target->dump_as_data( full_dump => $args{full_dump} // 0);
 
-    my $yaml = Dump $perl_data ;
+    my $yaml = Dump( $perl_data );
 
     $args{io_handle}->print($yaml);
 
@@ -110,7 +127,7 @@ Config::Model::Backend::Yaml - Read and write config as a YAML data structure
 
 =head1 VERSION
 
-version 2.097
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -135,13 +152,12 @@ version 2.097
             },
         },
     ],
-  read_config  => [
-                    { backend => 'yaml' , 
-                      config_dir => '/tmp',
-                      file  => 'foo.yml',
-                      auto_create => 1,
-                    }
-                  ],
+  rw_config  => {
+    backend => 'yaml',
+    config_dir => '/tmp',
+    file  => 'foo.yml',
+    auto_create => 1,
+  }
  ) ;
 
  my $inst = $model->instance(root_class_name => 'MyClass' );
@@ -220,6 +236,23 @@ would be written with:
  - un
  - deux
 
+=head1 backend parameter
+
+=head2 yaml_class
+
+By default, this module uses L<YAML::Tiny>. This module has the
+advantage of being light and
+L<secure|https://github.com/ingydotnet/yaml-libyaml-pm/issues/45>.  No
+Perl object can be created with YAML tags so L<YAML::Tiny> can be
+used with YAML files coming from unutrusted sources.
+
+On the other hand, L<YAML::Tiny> does not support boolean values: it
+cannot write C<true> and C<false> as plain scalar. C<true> and
+C<false> are quoted and are not of type boolean from YAML point of view.
+
+If this is a problem for your configuration files, you can use L<YAML>
+module which writes C<true> and C<false> without quotes.
+
 =head1 CONSTRUCTOR
 
 =head2 new ( node => $node_obj, name => 'yaml' ) ;
@@ -231,7 +264,7 @@ called by L<Config::Model::BackendMgr>.
 
 Of all parameters passed to this read call-back, only C<io_handle> is
 used. This parameter must be L<IO::File> object already opened for
-read. 
+read.
 
 It can also be undef. In which case C<read()> returns 0.
 
@@ -261,7 +294,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
