@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2017 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Exception;
-$Config::Model::Exception::VERSION = '2.106';
+$Config::Model::Exception::VERSION = '2.117';
 use warnings;
 use strict;
 use Data::Dumper;
@@ -100,19 +100,19 @@ sub full_message {
 }
 
 package Config::Model::Exception::Any;
-$Config::Model::Exception::Any::VERSION = '2.106';
+$Config::Model::Exception::Any::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception';
 
 package Config::Model::Exception::ModelDeclaration;
-$Config::Model::Exception::ModelDeclaration::VERSION = '2.106';
+$Config::Model::Exception::ModelDeclaration::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::Fatal';
 
 sub _desc {'configuration model declaration error' }
 
 package Config::Model::Exception::User ;
-$Config::Model::Exception::User::VERSION = '2.106';
+$Config::Model::Exception::User::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::Any';
 sub _desc {'user error' }
@@ -120,7 +120,7 @@ sub _desc {'user error' }
 
 ## old classes below
 package Config::Model::Exception::Syntax;
-$Config::Model::Exception::Syntax::VERSION = '2.106';
+$Config::Model::Exception::Syntax::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::Any';
 
@@ -141,7 +141,7 @@ sub full_message {
 }
 
 package Config::Model::Exception::LoadData;
-$Config::Model::Exception::LoadData::VERSION = '2.106';
+$Config::Model::Exception::LoadData::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::User';
 
@@ -165,7 +165,7 @@ sub full_message {
 }
 
 package Config::Model::Exception::Model;
-$Config::Model::Exception::Model::VERSION = '2.106';
+$Config::Model::Exception::Model::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::Fatal';
 
@@ -198,7 +198,7 @@ sub full_message {
 }
 
 package Config::Model::Exception::Load;
-$Config::Model::Exception::Load::VERSION = '2.106';
+$Config::Model::Exception::Load::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::User';
 
@@ -227,7 +227,7 @@ sub full_message {
 }
 
 package Config::Model::Exception::UnavailableElement;
-$Config::Model::Exception::UnavailableElement::VERSION = '2.106';
+$Config::Model::Exception::UnavailableElement::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::User';
 
@@ -258,7 +258,7 @@ sub full_message {
 }
 
 package Config::Model::Exception::AncestorClass;
-$Config::Model::Exception::AncestorClass::VERSION = '2.106';
+$Config::Model::Exception::AncestorClass::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::User';
 
@@ -266,7 +266,7 @@ sub _desc { 'unknown ancestor class'}
 
 
 package Config::Model::Exception::ObsoleteElement;
-$Config::Model::Exception::ObsoleteElement::VERSION = '2.106';
+$Config::Model::Exception::ObsoleteElement::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::User';
 
@@ -291,7 +291,7 @@ sub full_message {
 }
 
 package Config::Model::Exception::UnknownElement;
-$Config::Model::Exception::UnknownElement::VERSION = '2.106';
+$Config::Model::Exception::UnknownElement::VERSION = '2.117';
 use Carp;
 
 use Mouse;
@@ -357,7 +357,7 @@ sub full_message {
             $msg .= "\t"
                 . $parent->fetch_element(
                 name => $element_name,
-                qw/master check no accept_hidden 1/
+                qw/check no accept_hidden 1/
                 )->warp_error;
         }
     }
@@ -368,14 +368,14 @@ sub full_message {
 }
 
 package Config::Model::Exception::WarpError;
-$Config::Model::Exception::WarpError::VERSION = '2.106';
+$Config::Model::Exception::WarpError::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::User';
 
 sub _desc { 'warp error'}
 
 package Config::Model::Exception::Fatal;
-$Config::Model::Exception::Fatal::VERSION = '2.106';
+$Config::Model::Exception::Fatal::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::Any';
 
@@ -383,7 +383,7 @@ sub _desc { 'fatal error' }
 
 
 package Config::Model::Exception::UnknownId;
-$Config::Model::Exception::UnknownId::VERSION = '2.106';
+$Config::Model::Exception::UnknownId::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::User';
 
@@ -417,7 +417,7 @@ sub full_message {
 }
 
 package Config::Model::Exception::WrongValue;
-$Config::Model::Exception::WrongValue::VERSION = '2.106';
+$Config::Model::Exception::WrongValue::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::User';
 
@@ -425,7 +425,7 @@ sub _desc { 'wrong value'};
 
 
 package Config::Model::Exception::WrongType;
-$Config::Model::Exception::WrongType::VERSION = '2.106';
+$Config::Model::Exception::WrongType::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::User';
 
@@ -459,38 +459,40 @@ sub full_message {
 }
 
 package Config::Model::Exception::ConfigFile;
-$Config::Model::Exception::ConfigFile::VERSION = '2.106';
+$Config::Model::Exception::ConfigFile::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::User';
 
 sub _desc { 'error in configuration file' }
 
 package Config::Model::Exception::ConfigFile::Missing;
-$Config::Model::Exception::ConfigFile::Missing::VERSION = '2.106';
+$Config::Model::Exception::ConfigFile::Missing::VERSION = '2.117';
 use Mouse;
+use Mouse::Util::TypeConstraints;
+
 extends 'Config::Model::Exception::ConfigFile';
 
 sub _desc { 'missing configuration file'}
 
-has tried_files => (is => 'rw', isa => 'ArrayRef');
+subtype 'ExcpPathTiny', as 'Object', where {$_->isa('Path::Tiny')} ;
+
+has file => (is => 'rw', isa => 'Str | ExcpPathTiny' );
 
 sub full_message {
     my $self = shift;
 
-    my $msg = "Error: cannot find configuration file " . join (' or ', @{ $self->tried_files });
-
-    return $msg . "\n";
+    return "Error: cannot find configuration file " . $self->file . "\n";
 }
 
 package Config::Model::Exception::Formula;
-$Config::Model::Exception::Formula::VERSION = '2.106';
+$Config::Model::Exception::Formula::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::Model';
 
 sub _desc { 'error in computation formula of the configuration model'}
 
 package Config::Model::Exception::Internal;
-$Config::Model::Exception::Internal::VERSION = '2.106';
+$Config::Model::Exception::Internal::VERSION = '2.117';
 use Mouse;
 extends 'Config::Model::Exception::Fatal';
 
@@ -512,7 +514,7 @@ Config::Model::Exception - Exception mechanism for configuration model
 
 =head1 VERSION
 
-version 2.106
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -570,7 +572,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2017 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
