@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::HashId;
-$Config::Model::HashId::VERSION = '2.097';
+$Config::Model::HashId::VERSION = '2.117';
 use Mouse;
 use 5.10.1;
 
@@ -27,6 +27,7 @@ my $logger = get_logger("Tree::Element::Id::Hash");
 extends qw/Config::Model::AnyId/;
 
 with "Config::Model::Role::Grab";
+with "Config::Model::Role::ComputeFunction";
 
 has data => ( is => 'rw', isa => 'HashRef',  default => sub { {}; } );
 has list => (
@@ -45,7 +46,7 @@ has [qw/default_keys auto_create_keys/] => (
     coerce => 1,
     default => sub { []; }
 );
-has [qw/morph ordered/] => ( is => 'ro', isa => 'Bool' );
+has [qw/ordered write_empty_value/] => ( is => 'ro', isa => 'Bool', default => 0 );
 
 sub BUILD {
     my $self = shift;
@@ -92,7 +93,7 @@ sub set_properties {
     # warp mechanism
     foreach my $k ( sort keys %$data ) {
         next unless $wrong->($k);
-        $logger->debug( "set_properties: ", $self->name, " deleting id $k" );
+        $logger->trace( "set_properties: ", $self->name, " deleting id $k" );
         delete $data->{$k};
     }
 }
@@ -149,7 +150,6 @@ sub _fetch_all_indexes {
 # fetch without any check
 sub _fetch_with_id {
     my ( $self, $key ) = @_;
-    my $i = $self->instance;
     return $self->{data}{$key};
 }
 
@@ -158,6 +158,7 @@ sub _store {
     my ( $self, $key, $value ) = @_;
     push @{ $self->{list} }, $key
         unless exists $self->{data}{$key};
+    $self->notify_change(note => "added entry $key") if $self->write_empty_value;
     return $self->{data}{$key} = $value;
 }
 
@@ -477,7 +478,7 @@ sub load_data {
             @load_keys = @{ delete $data->{$order_key} or delete $data->{__order} };
             $from      = ' with '.$order_key;
         }
-        elsif ( $self->{ordered} ) {
+        elsif ( $self->{ordered} and (keys %$data > 1)) {
             $logger->warn( "HashId "
                     . $self->location
                     . ": loading ordered "
@@ -494,7 +495,7 @@ sub load_data {
                 . $from );
         foreach my $elt (@load_keys) {
             my $obj = $self->fetch_with_id($elt);
-            $obj->load_data( %args, data => $data->{$elt} );
+            $obj->load_data( %args, data => $data->{$elt} ) if defined $data->{$elt};
         }
     }
     elsif ( ref($data) eq 'ARRAY' ) {
@@ -538,7 +539,7 @@ Config::Model::HashId - Handle hash element for configuration model
 
 =head1 VERSION
 
-version 2.097
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -656,7 +657,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
