@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::IdElementReference;
-$Config::Model::IdElementReference::VERSION = '2.082';
+$Config::Model::IdElementReference::VERSION = '2.117';
 use Mouse;
 
 use Carp;
@@ -82,7 +82,7 @@ sub get_choice_from_refered_to {
 
         my @path = split( /\s+/, $user_spec );
 
-        $logger->debug("path: @path");
+        $logger->trace("path: @path");
 
         my $refered_to = eval { $config_elt->grab("@path"); };
 
@@ -190,7 +190,7 @@ Config::Model::IdElementReference - Refer to id element(s) and extract keys
 
 =head1 VERSION
 
-version 2.082
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -271,10 +271,10 @@ version 2.082
  my $root = $inst->config_root ;
 
  # configure hosts on my network
- my $step = 'host:foo ip_nic:eth0=192.168.0.1 ip_nic:eth1=192.168.1.1 -
+ my $steps = 'host:foo ip_nic:eth0=192.168.0.1 ip_nic:eth1=192.168.1.1 -
              host:bar ip_nic:eth0=192.168.0.2 ip_nic:eth1=192.168.1.2 -
              host:baz ip_nic:eth0=192.168.0.3 ip_nic:eth1=192.168.1.3 ';
- $root->load( step => $step );
+ $root->load( steps => $steps );
 
  print "master host can be one of ",
    join(' ',$root->fetch_element('master_host')->get_choice),"\n" ; 
@@ -314,10 +314,10 @@ Construction is handled by the calling object (L<Config::Model::Node>).
 
 =item refer_to
 
-C<refer_to> is used to specify a hash element that will be used as a
+C<refer_to> is used to specify a hash element that is used as a
 reference. C<refer_to> points to an array or hash element in the
 configuration tree using the path syntax (See
-L<Config::Model::Node/grab> for details).
+L<Config::Model::Role::Grab/grab> for details).
 
 =item computed_refer_to
 
@@ -378,7 +378,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
