@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2017 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::ListId;
-$Config::Model::ListId::VERSION = '2.106';
+$Config::Model::ListId::VERSION = '2.117';
 use 5.10.1;
 use Mouse;
 
@@ -366,11 +366,15 @@ sub _assert_leaf_cargo {
     ) unless $ct eq 'leaf';
 }
 
+sub sort_algorithm {
+    return sub { $_[0]->fetch cmp $_[1]->fetch; };
+}
+
 sub sort {
     my $self = shift;
 
     $self->_assert_leaf_cargo;
-    $self->_sort_data( sub { $_[0]->fetch cmp $_[1]->fetch; } );
+    $self->_sort_data( $self->sort_algorithm );
 
     my $has_changed = $self->_reindex;
     $self->notify_change( note => "sorted" ) if $has_changed;
@@ -505,7 +509,7 @@ Config::Model::ListId - Handle list element for configuration model
 
 =head1 VERSION
 
-version 2.106
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -649,6 +653,12 @@ For instance
 
 loads C< [ 'foo','bar']> in C<$elt>
 
+=head2 sort_algorithm
+
+Returns a sub used to sort the list elements. See
+L<perlfunc/sort>. Used only for list of leaves. This method can be
+overridden to alter sort order.
+
 =head1 AUTHOR
 
 Dominique Dumont, (ddumont at cpan dot org)
@@ -667,7 +677,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2017 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
