@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Lister;
-$Config::Model::Lister::VERSION = '2.082';
+$Config::Model::Lister::VERSION = '2.117';
 use strict;
 use warnings;
 use Exporter;
@@ -36,7 +36,14 @@ sub available_models {
             next if $file =~ m!/README!;
             next if $file =~ /(~|\.bak|\.orig)$/;
             my ($appli) = ( $file =~ m!.*/([\w\-]+)! );
+
+            # ensure that an appli file of a cat is not parsed twice
+            # (useful in dev, where system appli file may clobber
+            # appli file in dvelopment
+            next if $done_cat{$cat}{$appli};
+
             $appli_info{$appli}{_file} = $file;
+            $appli_info{$appli}{_category} = $cat;
             open( F, $file ) || die "Can't open file $file:$!";
             while (<F>) {
                 chomp;
@@ -53,6 +60,7 @@ sub available_models {
                 $appli_info{$appli}{$k} = $v;
                 $applications{$appli} = $v if $k =~ /model/i;
             }
+            die "Missing model line in file $file\n" unless $done_cat{$cat}{$appli};
         }
     }
     return \%categories, \%appli_info, \%applications;
@@ -84,7 +92,7 @@ Config::Model::Lister - List available models and applications
 
 =head1 VERSION
 
-version 2.082
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -143,7 +151,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
