@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Loader;
-$Config::Model::Loader::VERSION = '2.082';
+$Config::Model::Loader::VERSION = '2.117';
 use Carp;
 use strict;
 use warnings;
@@ -35,8 +35,10 @@ sub load {
 
     croak "load error: missing 'node' parameter" unless defined $node;
 
-    my $step = delete $args{step};
-    croak "load error: missing 'step' parameter" unless defined $step;
+    my $steps = delete $args{steps} // delete $args{step};
+    croak "load error: missing 'steps' parameter" unless defined $steps;
+
+    my $caller_is_root = delete $args{caller_is_root};
 
     if (delete $args{experience}) {
         carp "load: experience parameter is deprecated";
@@ -49,7 +51,7 @@ sub load {
     croak __PACKAGE__, "load: unexpected check $check" unless $check =~ /yes|no|skip/;
 
     # accept commands
-    my $huge_string = ref $step ? join( ' ', @$step ) : $step;
+    my $huge_string = ref $steps ? join( ' ', @$steps ) : $steps;
 
     # do a split on ' ' but take quoted string into account
     my @command = (
@@ -78,12 +80,14 @@ sub load {
     my $ret;
     do {
         $ret = $self->_load( $current_node, $check, \@command, 1 );
-        $logger->debug("_load returned $ret");
+        $logger->trace("_load returned $ret");
 
         # found '!' command
         if ( $ret eq 'root' ) {
-            $logger->debug("Setting current_node to root node");
-            $current_node = $current_node->root;
+            $current_node = $caller_is_root ? $node : $current_node->root;
+            if ($logger->debug) {
+                $logger->debug("Setting current_node to root node: ".$current_node->name);
+            }
         }
     } while ( $ret eq 'root' );
 
@@ -113,6 +117,7 @@ sub load {
 # returns elt action id subaction value
 sub _split_cmd {
     my $cmd = shift;
+    $logger->trace("split on: ->$cmd<-");
 
     my $quoted_string = qr/"(?: \\" | [^"] )* "/x;    # quoted string
 
@@ -123,7 +128,7 @@ sub _split_cmd {
 	 (?:
             (:~|:-[=~]?|:=~|:\.\w+|:[=<>@]?|~)       # action
             (?:
-                  (?: \( ( $quoted_string | [^)]+ ) \) )  # capture parameters between braces
+                  (?: \( ( $quoted_string | [^)]+ ) \) )  # capture parameters between ( )
                 | (
                     /[^/]+/      # regexp
                     | (?:
@@ -132,16 +137,19 @@ sub _split_cmd {
                       )+
                   )
             )?
-         )?
+     )?
 	 (?:
-            (=~|.=|[=<>])          # apply regexp or assign or append
-	    (
-              (?:
-                $quoted_string
-                | [^#\s]                # or non whitespace
-              )+                       # many
-            )
-	 )?
+            (=~|\.=|=\.\w+|[=<>])          # apply regexp or assign or append
+            (?:
+                  (?: \( ( $quoted_string | [^)]+ ) \) )  # capture parameters between ( )
+                | (
+                    (?:
+                      $quoted_string
+                     | [^#\s]                # or non whitespace
+                    )+                       # many
+                  )
+	        )?
+     )?
 	 (?:
             \#              # optional annotation
 	    (
@@ -151,9 +159,18 @@ sub _split_cmd {
               )+                       # many
             )
 	 )?
-         !gx
+     (.*)    # leftover
+    !gx
     );
 
+    my $leftout = pop @command;
+
+    if ($leftout) {
+        Config::Model::Exception::Load->throw(
+            command => $cmd,
+            error   => "Syntax error: spurious char at command end: '$leftout'. Did you forget double quotes ?"
+        );
+    }
     return wantarray ? @command : \@command;
 }
 
@@ -171,7 +188,7 @@ sub _load {
     my ( $self, $node, $check, $cmdref, $at_top_level ) = @_;
     $at_top_level ||= 0;
     my $node_name = "'" . $node->name . "'";
-    $logger->debug("_load: called on node $node_name");
+    $logger->trace("_load: called on node $node_name");
 
     my $inst = $node->instance;
 
@@ -209,13 +226,16 @@ sub _load {
             }
         }
 
-        my @instructions = _split_cmd($cmd);
-        my ( $element_name, $action, $function_param, $id, $subaction, $value, $note ) =
-            @instructions;
+        my ( $element_name, $action, $function_param, $id, $subaction, $value_function_param2, $value_param, $note ) =
+            _split_cmd($cmd);
+
+        # regexp ensure that only $value_function_param  $value_param is set
+        my $value = $value_function_param2 // $value_param ;
+        my @instructions = ( $element_name, $action, $function_param, $id, $subaction, $value, $note );
 
         if ( $logger->is_debug ) {
             my @disp = map { defined $_ ? "'$_'" : '<undef>' } @instructions;
-            $logger->debug("_load instructions: @disp");
+            $logger->debug("_load instructions: @disp (from: $cmd)");
         }
 
         if ( not defined $element_name and not defined $note ) {
@@ -399,49 +419,71 @@ sub _load_check_list {
 # function_args are the arguments passed to the load command
 my %dispatch_action = (
     list_leaf => {
-        ':.sort'          => sub { $_[1]->sort; },
-        ':.push'          => sub { $_[1]->push( @_[ 4 .. $#_ ] ); },
-        ':.unshift'       => sub { $_[1]->unshift( @_[ 4 .. $#_ ] ); },
-        ':.insert_at'     => sub { $_[1]->insert_at( @_[ 4 .. $#_ ] ); },
-        ':.insort'        => sub { $_[1]->insort( @_[ 4 .. $#_ ] ); },
+        ':.sort'          => sub { $_[1]->sort;  return 'ok';},
+        ':.push'          => sub { $_[1]->push( @_[ 5 .. $#_ ] ); return 'ok'; },
+        ':.unshift'       => sub { $_[1]->unshift( @_[ 5 .. $#_ ] ); return 'ok'; },
+        ':.insert_at'     => sub { $_[1]->insert_at( @_[ 5 .. $#_ ] ); return 'ok'; },
+        ':.insort'        => sub { $_[1]->insort( @_[ 5 .. $#_ ] ); return 'ok'; },
         ':.insert_before' => \&_insert_before,
     },
-    'hash_*' => {
-        ':.sort'          => sub { $_[1]->sort; },
-        ':@'              => sub { $_[1]->sort; },
-        ':.copy'          => sub { $_[1]->copy( $_[4], $_[5] ); },
+    'list_*' => {
+        ':.copy'          => sub { $_[1]->copy( $_[5], $_[6] ); return 'ok'; },
+        ':.clear'         => sub { $_[1]->clear; return 'ok'; },
     },
+    hash_leaf => {
+        ':.insort'        => sub { $_[1]->insort($_[5])->store($_[6]); return 'ok'; },
+    },
+    hash_node =>  => {
+        ':.insort'        => \&_insort_hash_of_node,
+    },
+    'hash_*' => {
+        ':.sort'          => sub { $_[1]->sort; return 'ok'; },
+        ':@'              => sub { $_[1]->sort; return 'ok'; },
+        ':.copy'          => sub { $_[1]->copy( $_[5], $_[6] ); return 'ok'; },
+        ':.clear'         => sub { $_[1]->clear; return 'ok';},
+    },
+    # part of list or hash. leaf element have their own dispatch table
+    # (%load_value_dispatch) because the signture of the sub ref are
+    # different between the 2 dispatch tables.
     leaf => {
-        ':-=' => \&_remove_by_value,
-        ':-~' => \&_remove_matched_value,
-        ':=~' => \&_substitute_value,
+        ':.rm_value' => \&_remove_by_value,
+        ':.rm_match' => \&_remove_matched_value,
+        ':.subtitute' => \&_substitute_value,
     },
     fallback => {
-        ':-' => \&_remove_by_id,
-        '~'  => \&_remove_by_id,
-    } );
+        ':.rm' => \&_remove_by_id,
+    }
+);
 
-my @equiv = qw/:@ :.sort :< :.push :> :.unshift/;
-while (@equiv) {
-    my ( $to, $from ) = splice @equiv, 0, 2;
-    $dispatch_action{list_leaf}{$to} = $dispatch_action{list_leaf}{$from};
+my %equiv = (
+    list_leaf => { qw/:@ :.sort :< :.push :> :.unshift/ },
+    # fix for cme gh#2
+    leaf => { qw/:-= :.rm_value :-~ :.rm_match :=~ :.subtitute/ },
+    fallback => { qw/:- :.rm ~ :.rm/ },
+);
+
+while ( my ($target, $sub_equiv) = each %equiv) {
+    while ( my ($new_action, $existing_action) = each %$sub_equiv) {
+        $dispatch_action{$target}{$new_action} = $dispatch_action{$target}{$existing_action};
+    }
 }
 
 sub _insert_before {
-    my ( $self, $element, $check, $inst, $before_str, @values ) = @_;
+    my ( $self, $element, $check, $inst, $cmdref, $before_str, @values ) = @_;
     my $before = $before_str =~ m!^/! ? eval "qr$before_str" : $before_str;
     $element->insert_before( $before, @values );
+    return 'ok';
 }
 
 sub _remove_by_id {
-    my ( $self, $element, $check, $inst, $id ) = @_;
-    $logger->debug("_remove_by_id: removing id $id");
+    my ( $self, $element, $check, $inst, $cmdref, $id ) = @_;
+    $logger->debug("_remove_by_id: removing id '$id'");
     $element->remove($id);
     return 'ok';
 }
 
 sub _remove_by_value {
-    my ( $self, $element, $check, $inst, $rm_val ) = @_;
+    my ( $self, $element, $check, $inst, $cmdref, $rm_val ) = @_;
 
     $logger->debug("_remove_by_value value $rm_val");
     foreach my $idx ( $element->fetch_all_indexes ) {
@@ -453,7 +495,7 @@ sub _remove_by_value {
 }
 
 sub _remove_matched_value {
-    my ( $self, $element, $check, $inst, $rm_val ) = @_;
+    my ( $self, $element, $check, $inst, $cmdref, $rm_val ) = @_;
 
     $logger->debug("_remove_matched_value $rm_val");
 
@@ -468,7 +510,7 @@ sub _remove_matched_value {
 }
 
 sub _substitute_value {
-    my ( $self, $element, $check, $inst, $s_val ) = @_;
+    my ( $self, $element, $check, $inst, $cmdref, $s_val ) = @_;
 
     $logger->debug("_substitute_value $s_val");
 
@@ -480,6 +522,13 @@ sub _substitute_value {
     return 'ok';
 }
 
+sub _insort_hash_of_node {
+    my ( $self, $element, $check, $inst, $cmdref, $id ) = @_;
+    my $node = $element->insort($_[5]);
+    $logger->debug("_insort_hash_of_node: calling _load on node id $id");
+    return $self->_load( $node, $check, $cmdref );
+}
+
 sub _load_list {
     my ( $self, $node, $check, $inst, $cmdref ) = @_;
     my ( $element_name, $action, $f_arg, $id, $subaction, $value, $note ) = @$inst;
@@ -529,8 +578,7 @@ sub _load_list {
             || $dispatch_action{$cargo_type}{$action}
             || $dispatch_action{'fallback'}{$action};
         if ($dispatch) {
-            $dispatch->( $self, $element, $check, $inst, @f_args );
-            return 'ok';
+            return $dispatch->( $self, $element, $check, $inst, $cmdref, @f_args );
         }
     }
 
@@ -593,17 +641,19 @@ sub _load_hash {
         Config::Model::Exception::Load->throw(
             object  => $element,
             command => join( '', map { $_ || '' } @$inst ),
-            error   => "Missing assignment on " . "element type: hash, cargo_type: $cargo_type"
+            error   => "Missing key (e.g. '$element_name:some_key') on hash element, cargo_type: $cargo_type"
         );
     }
 
-    if ( $action eq ':~' ) {
+    # loop requires $subaction so does not fit in the dispath table
+    if ( $action eq ':~' or $action eq ':.foreach_match' ) {
         my @keys = $element->fetch_all_indexes;
         my $ret  = 'ok';
-        $id =~ s!^/|/$!!g if $id;
-        my @loop_on = $id ? grep { /$id/ } @keys : @keys;
+        my $pattern = $id // $f_arg;
+        $pattern =~ s!^/|/$!!g if $pattern;
+        my @loop_on = $pattern ? grep { /$pattern/ } @keys : @keys;
         if ($logger->is_debug) {
-            my $str = $id ? " with regex /$id/" : '';
+            my $str = $pattern ? " with regex /$pattern/" : '';
             $logger->debug("_load_hash: looping$str on keys @loop_on");
         }
 
@@ -645,11 +695,18 @@ sub _load_hash {
             || $dispatch_action{'fallback'}{$action};
         if ($dispatch) {
             # todo missing arguments
-            $dispatch->( $self, $element, $check, $inst, @f_args );
-            return 'ok';
+            return $dispatch->( $self, $element, $check, $inst, $cmdref, @f_args );
         }
     }
 
+    if (not defined $id) {
+        Config::Model::Exception::Load->throw(
+            object  => $element,
+            command => join( '', @$inst ),
+            error   => qq!Unexpected hash instruction: '$action' or missing id!
+        );
+    }
+
     my $obj = $element->fetch_with_id( index => $id, check => $check );
     $self->_load_note( $obj, $note, $inst, $cmdref );
 
@@ -672,16 +729,16 @@ sub _load_hash {
         $self->_load_value( $obj, $check, $subaction, $value )
             and return 'ok';
     }
-    elsif ( $action eq ':' and defined $note ) {
-
-        # action was just to store annotation
+    elsif ( $action eq ':' ) {
+        $logger->debug("_load_hash: created empty element of type $cargo_type");
         return 'ok';
     }
     elsif ($action) {
+        $logger->debug("_load_hash: giving up");
         Config::Model::Exception::Load->throw(
             object  => $element,
             command => join( '', grep { defined $_ } @$inst ),
-            error   => "Hash assignment with '$action' on unexpected " . "cargo_type: $cargo_type"
+            error   => "Hash load with '$action' on unexpected " . "cargo_type: $cargo_type"
         );
     }
 }
@@ -741,33 +798,84 @@ sub _load_leaf {
     );
 }
 
-sub _load_value {
-    my ( $self, $element, $check, $subaction, $value, $inst ) = @_;
+# sub is called with  ( $self, $element, $value, $check, $instructions )
+# function_args are the arguments passed to the load command
+my %load_value_dispatch = (
+    '=' => sub { $_[1]->store( value => $_[2], check => $_[3] ); return 'ok'; },
+    '.=' => \&_append_value,
+    '=~' => \&_apply_regexp_on_value,
+    '=.file' => \&_store_file_in_value,
+    '=.env' => sub { $_[1]->store( value => $ENV{$_[2]}, check => $_[3] ); return 'ok'; },
+);
 
-    $logger->debug("_load_value: action '$subaction' value '$value' check $check");
-    if ( $subaction eq '=' and $element->isa('Config::Model::Value') ) {
-        $element->store( value => $value, check => $check );
+sub _append_value {
+    my ( $self, $element, $value, $check, $instructions ) = @_;
+    my $orig = $element->fetch( check => $check );
+    $element->store( value => $orig . $value, check => $check );
+}
+
+sub _apply_regexp_on_value {
+    my ( $self, $element, $value, $check, $instructions ) = @_;
+
+    my $orig = $element->fetch( check => $check );
+    return unless defined $orig;
+
+    eval("\$orig =~ $value;");
+    if ($@) {
+        Config::Model::Exception::Load->throw(
+            object  => $element,
+            command => $instructions,
+            error => "Failed regexp '$value' on " . "element '"
+                . $element->name . "' : $@"
+        );
     }
-    elsif ( $subaction eq '.=' and $element->isa('Config::Model::Value') ) {
-        my $orig = $element->fetch( check => $check );
-        $element->store( value => $orig . $value, check => $check );
+    $element->store( value => $orig, check => $check );
+}
+
+sub _store_file_in_value {
+    my ( $self, $element, $value, $check, $instructions ) = @_;
+
+    if ($value eq '-') {
+        $element->store( value => join('',<STDIN>), check => $check );
+        return 'ok';
     }
-    elsif ( $subaction eq '=~' and $element->isa('Config::Model::Value') ) {
-        my $orig = $element->fetch( check => $check );
-        if ( defined $orig ) {
-            eval("\$orig =~ $value;");
-            if ($@) {
-                Config::Model::Exception::Load->throw(
-                    object  => $element,
-                    command => $inst,
-                    error => "Failed regexp '$value' on " . "element '" . $element->name . "' : $@"
-                );
-            }
-            $element->store( value => $orig, check => $check );
-        }
+
+    my $path = $element->root_path->child($value);
+    if ($path->is_file) {
+        $element->store( value => $path->slurp_utf8, check => $check );
     }
     else {
-        return undef;
+        Config::Model::Exception::Load->throw(
+            object  => $element,
+            command => $instructions,
+            error => "cannot read file $value"
+        );
+    }
+}
+
+sub _load_value {
+    my ( $self, $element, $check, $subaction, $value, $instructions ) = @_;
+
+    if (not $element->isa('Config::Model::Value')) {
+        my $class = ref($element);
+        Config::Model::Exception::Load->throw(
+            object  => $element,
+            command => $instructions,
+            error   => "Load error: _load_value called on non Value object. ($class)"
+        );
+    }
+
+    $logger->debug("_load_value: action '$subaction' value '$value' check $check");
+    my $dispatch = $load_value_dispatch{$subaction};
+    if ($dispatch) {
+        return $dispatch->( $self, $element, $value, $check, $instructions );
+    }
+    else {
+        Config::Model::Exception::Load->throw(
+            object  => $element,
+            command => $instructions,
+            error => "Unexpected operator or function on value: $subaction"
+            );
     }
 
     $logger->debug("_load_value: done returns ok");
@@ -790,7 +898,7 @@ Config::Model::Loader - Load serialized data into config tree
 
 =head1 VERSION
 
-version 2.082
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -839,11 +947,11 @@ version 2.082
  my $root = $inst->config_root ;
 
  # put data
- my $step = 'foo=FOO hash_of_nodes:fr foo=bonjour -
+ my $steps = 'foo=FOO hash_of_nodes:fr foo=bonjour -
    hash_of_nodes:en foo=hello
    ! lista=foo,bar lista:2=baz
      listb:0=foo listb:1=baz';
- $root->load( step => $step );
+ $root->load( steps => $steps );
 
  print $root->describe,"\n" ;
  # name         value        type         comment
@@ -855,14 +963,14 @@ version 2.082
 
 
  # delete some data
- $root->load( step => 'lista~2' );
+ $root->load( steps => 'lista~2' );
 
  print $root->describe(element => 'lista'),"\n" ;
  # name         value        type         comment
  # lista        foo,bar      list
 
  # append some data
- $root->load( step => q!hash_of_nodes:en foo.=" world"! );
+ $root->load( steps => q!hash_of_nodes:en foo.=" world"! );
 
  print $root->grab('hash_of_nodes:en')->describe(element => 'foo'),"\n" ;
  # name         value        type         comment
@@ -939,10 +1047,10 @@ command specified below, e.g C</a_string="foo bar">
 =item xxx:yy
 
 Go down using C<xxx> element and id C<yy> (For C<hash> or C<list>
-element with C<node> cargo_type). Literal C<\n> will be replaced by
+element with C<node> cargo_type). Literal C<\n> are replaced by
 real C<\n> (LF in Unix).
 
-=item xxx:~yy
+=item xxx:.foreach_match(yy) or xxx:~yy
 
 Go down using C<xxx> element and loop over the ids that match the regex
 specified by C<yy>. (For C<hash>).
@@ -957,8 +1065,8 @@ The leading and trailing '/' may be omitted. Be sure to surround the
 regexp with double quote if space are embedded in the regex.
 
 Note that the loop ends when the load command goes above the element
-where the loop is executed. For instance, the instruction below will
-try to execute C<DX=BV> and C<int_v=9> for all elements of C<std_id> hash:
+where the loop is executed. For instance, the instruction below
+tries to execute C<DX=BV> and C<int_v=9> for all elements of C<std_id> hash:
 
  std_id:~/^\w+$/ DX=Bv int_v=9
 
@@ -970,24 +1078,24 @@ In the examples below only C<DX=BV> is executed by the loop:
 The loop is done on all elements of the hash when no value is passed
 after "C<:~>" (mnemonic: an empty regexp matches any value).
 
-=item xxx:-yy
+=item xxx:.rm(yy) or xxx:-yy
 
 Delete item referenced by C<xxx> element and id C<yy>. For a list,
 this is equivalent to C<splice xxx,yy,1>. This command does not go
 down in the tree (since it has just deleted the element). I.e. a
 'C<->' is generally not needed afterwards.
 
-=item xxx:-=yy
+=item xxx:.rm_value(yy) or xxx:-=yy
 
 Remove the element whose value is C<yy>. For list or hash of leaves.
-Will not complain if the value to delete is not found.
+Does not not complain if the value to delete is not found.
 
-=item xxx:-~/yy/
+=item xxx:..rm_match(yy) or xxx:-~/yy/
 
 Remove the element whose value matches C<yy>. For list or hash of leaves.
-Will not complain if no value were deleted.
+Does not not complain if no value were deleted.
 
-=item xxx:=~s/yy/zz/
+=item xxx:.substitute(/yy/zz/) or xxx=~s/yy/zz/
 
 Substitute a value with another. Perl switches can be used(e.g. C<xxx:=~s/yy/zz/gi>)
 
@@ -1019,6 +1127,23 @@ Insert C<zz> value on C<xxx> list before B<value> matching C<yy>.
 
 Insert C<zz> value on C<xxx> list so that existing alphanumeric order is preserved.
 
+=item xxx:.insort(zz)
+
+For hash element containing nodes: creates a new hash element with
+C<zz> key on C<xxx> hash so that existing alphanumeric order of keys
+is preserved. Note that all keys are sorted once this instruction is
+called. Following instructions are applied on the created
+element. I.e. putting key order aside, C<xxx:.insort(zz)> has the
+same effect as C<xxx:zz> instruction.
+
+=item xxx:.insort(zz,vv)
+
+For hash element containing leaves: creates a new hash element with
+C<zz> key and assing value C<vv> so that existing alphanumeric order of keys
+is preserved. Note that all keys are sorted once this instruction is
+called. Putting key order aside, C<xxx:.insort(zz,vv)> has the
+same effect as C<xxx:zz=vv> instruction.
+
 =item xxx:=z1,z2,z3
 
 Set list element C<xxx> to list C<z1,z2,z3>. Use C<,,> for undef
@@ -1035,7 +1160,11 @@ Using C<xxx:~/yy/=zz> is also possible.
 
 =item xxx:.copy(yy,zz)
 
-copy item C<yy> in C<zz>
+copy item C<yy> in C<zz> (hash or list).
+
+=item xxx:.clear
+
+Clear the hash or list.
 
 =back
 
@@ -1046,8 +1175,8 @@ copy item C<yy> in C<zz>
 =item xxx=zz
 
 Set element C<xxx> to value C<yy>. load also accepts to set elements
-with a quoted string. (For C<leaf> element) Literal C<\n> will be replaced by
-real C<\n> (LF in Unix). Literal C<\\> will be replaced by C<\>.
+with a quoted string. (For C<leaf> element) Literal C<\n> are replaced by
+real C<\n> (LF in Unix). Literal C<\\> are replaced by C<\>.
 
 For instance C<foo="a quoted string"> or C<foo="\"bar\" and \"baz\"">.
 
@@ -1056,7 +1185,7 @@ For instance C<foo="a quoted string"> or C<foo="\"bar\" and \"baz\"">.
 Apply the substitution to the value of xxx. C<s/foo/bar/> is the standard Perl C<s>
 substitution pattern.
 
-If your patten needs white spaces, you will need to surround the pattern with quotes:
+Patterns with white spaces must be surrounded by quotes:
 
   xxx=~"s/foo bar/bar baz/"
 
@@ -1070,7 +1199,17 @@ Undef element C<xxx>
 
 =item xxx.=zzz
 
-Will append C<zzz> value to current values (valid for C<leaf> elements).
+Appends C<zzz> value to current value (valid for C<leaf> elements).
+
+=item xxx=.file(yyy)
+
+Store the content of file C<yyy> in element C<xxx>.
+
+Store STDIn in value xxx when C<yyy> is '-'.
+
+=item xxx=.env(yyy)
+
+Store the content of environment variable C<yyy> in element C<xxx>.
 
 =back
 
@@ -1132,7 +1271,7 @@ Now that ssh config is mucked up with dummy entries, let's clean up:
 
 Load data into the node tree (from the node passed with C<node>)
 and fill values as we go following the instructions passed with
-C<step>.  (C<step> can also be an array ref).
+C<steps>.  (C<steps> can also be an array ref).
 
 Parameters are:
 
@@ -1142,15 +1281,20 @@ Parameters are:
 
 node ref of the root of the tree (of sub-root) to start the load from.
 
-=item step
+=item steps (or step)
 
-A string or an array ref containing the steps to load. See above for a
-description of the string.
+A string or an array ref containing the steps to load. See
+L<above/"load string syntax"> for a description of the string.
 
 =item check
 
 Whether to check values while loading. Either C<yes> (default), C<no> or C<skip>.
-Loading with C<skip> will discard bad values.
+Bad values are discarded when C<check> is set to C<skip>.
+
+=item caller_is_root
+
+Change the target of the C<!> command: when set, the C<!> command go
+to caller node instead of going to root node. (default is false)
 
 =back
 
@@ -1168,7 +1312,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
