@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Node;
-$Config::Model::Node::VERSION = '2.088';
+$Config::Model::Node::VERSION = '2.117';
 use Mouse;
 with "Config::Model::Role::NodeLoader";
 
@@ -29,6 +29,10 @@ use List::MoreUtils qw(insert_after_string);
 
 extends qw/Config::Model::AnyThing/;
 
+with "Config::Model::Role::Grab";
+with "Config::Model::Role::HelpAsText";
+with "Config::Model::Role::ComputeFunction";
+
 use vars qw(@status @level %default_property);
 
 *status           = *Config::Model::status;
@@ -43,6 +47,8 @@ my %legal_properties = (
 my $logger     = get_logger("Tree::Node");
 my $fix_logger = get_logger("Anything::Fix");
 my $change_logger = get_logger("ChangeTracker");
+my $deep_check_logger = get_logger('DeepCheck');
+my $user_logger = get_logger('User');
 
 # Here are the legal element types
 my %create_sub_for = (
@@ -76,9 +82,29 @@ has initialized => ( is => 'rw', isa => 'Bool', default => 0 );
 
 has config_class_name => ( is => 'ro', isa => 'Str', required => 1 );
 
+has gist => (
+    is => 'rw',
+    isa => 'Str',
+    default => '',
+);
+
+sub fetch_gist {
+    my $self = shift;
+    my $gist = $self->gist // '';
+    $gist =~ s!{([\w -]+)}!$self->grab($1)->fetch // ''!ge;
+    return $gist;
+}
+
 has [qw/config_file element_name/] => ( is => 'ro', isa => 'Maybe[Str]', required => 0 );
 
-has instance => ( is => 'ro', isa => 'Config::Model::Instance', weak_ref => 1, required => 1 );
+has instance => (
+    is => 'ro',
+    isa => 'Config::Model::Instance',
+    weak_ref => 1,
+    required => 1,
+    handles => [qw/read_check/],
+);
+
 has config_model => (
     is       => 'ro',
     isa      => 'Config::Model',
@@ -92,8 +118,6 @@ sub _config_model {
     my $p    = $self->instance->config_model;
 }
 
-has skip_read  => ( is => 'ro', isa => 'Bool' );
-has check      => ( is => 'ro', isa => 'Str', default => 'yes' );
 has model      => ( is => 'rw', isa => 'HashRef' );
 has needs_save => ( is => 'rw', isa => 'Bool', default => 0 );
 
@@ -110,14 +134,6 @@ sub _backend_support_annotation {
 sub BUILD {
     my $self = shift;
 
-    my $read_check = $self->instance->read_check;
-
-    my $req_check = $self->check;
-    my $check =
-          $req_check eq 'no'   || $read_check eq 'no'   ? 'no'
-        : $req_check eq 'skip' || $read_check eq 'skip' ? 'skip'
-        :                                                 'yes';
-
     my $caller_class = defined $self->parent ? $self->parent->name : 'user';
 
     my $class_name = $self->config_class_name;
@@ -192,7 +208,6 @@ sub create_node {
         config_class_name => $config_class_name,
         instance          => $self->{instance},
         element_name      => $element_name,
-        check             => $check,
         parent            => $self,
         container         => $self,
     );
@@ -312,15 +327,14 @@ sub check_properties {
 
 sub init {
     my $self = shift;
+    my %args = @_;
 
     return if $self->{initialized};
     $self->{initialized} = 1;    # avoid recursions
 
     my $model = $self->{model};
 
-    return
-        unless defined $model->{read_config}
-        or defined $model->{write_config};
+    return unless defined $model->{rw_config};
 
     my $initial_load_backup = $self->instance->initial_load;
     $self->instance->initial_load_start;
@@ -329,24 +343,12 @@ sub init {
         # config_dir spec given by application info
         config_dir      => $self->instance->config_dir,
         node => $self,
+        rw_config => $model->{rw_config}
     );
 
-    if ( defined $model->{read_config} and not $self->skip_read ) {
-        $self->read_config_data( check => $self->check );
-    }
-
-    # use read_config data if write_config is missing
-    $model->{write_config} ||= dclone $model->{read_config}
-        if defined $model->{read_config};
-
-    if ( $model->{write_config} ) {
-
-        # setup auto_write, write_config_dir is obsolete
-        $self->backend_mgr->auto_write_init(
-            write_config     => $model->{write_config},
-            write_config_dir => $model->{write_config_dir},
-        );
-    }
+    $self->read_config_data( check => $self->read_check );
+    # setup auto_write
+    $self->backend_mgr->auto_write_init();
 
     $self->instance->initial_load($initial_load_backup);
 }
@@ -361,12 +363,10 @@ sub read_config_data {
             $self->location, ")\n";
     }
 
-    # setup auto_read, read_config_dir is obsolete
+    # setup auto_read
     # may use an overridden config file
     $self->backend_mgr->read_config_data(
-        read_config     => $model->{read_config},
         check           => $args{check},
-        read_config_dir => $model->{read_config_dir},
         config_file     => $args{config_file} || $self->{config_file},
         auto_create     => $args{auto_create} || $self->instance->auto_create,
     );
@@ -376,14 +376,14 @@ sub notify_change {
     my $self = shift;
     my %args = @_;
 
-    $change_logger->debug( "called for ", $self->name, " from ", join( ' ', caller ),
-        " with ", join( ' ', %args ) )
-        if $change_logger->is_debug;
-
+    if ($change_logger->is_trace) {
+        my @with = map { "'$_' -> '". ($args{$_} // '<undef>') ."'"  } sort keys %args;
+        $change_logger->trace("called for ", $self->name, " from ", join( ' ', caller ), " with ", join( ' ', @with ));
+    }
     return if $self->instance->initial_load and not $args{really};
 
-    $logger->debug( "called while needs_write is ", $self->needs_save, " for ", $self->name )
-        if $logger->is_debug;
+    $logger->trace( "called while needs_write is ", $self->needs_save, " for ", $self->name )
+        if $logger->is_trace;
 
     if ( defined $self->backend_mgr ) {
         $self->needs_save(1);    # will trigger a save in config_file
@@ -395,18 +395,6 @@ sub notify_change {
     }
 }
 
-sub write_back {
-    my ( $self, %args ) = @_;
-
-    my $force_write = delete $args{force} || 0;
-
-    if ( $self->location and $args{config_file} ) {
-        die "write_back: cannot override config_file in non root node (", $self->location, ")\n";
-    }
-
-    $self->backend_mgr->write_back(%args) if $self->needs_save or $force_write;
-}
-
 sub is_auto_write_for_type {
     my $self = shift;
     return 0 unless defined $self->backend_mgr;
@@ -511,13 +499,18 @@ sub get_element_names {
     my $type       = $args{type};              # optional
     my $cargo_type = $args{cargo_type};        # optional
 
-    $self->init;
+    $self->init();
 
     my @result;
 
     my $info         = $self->{model};
     my @element_list = @{ $self->{model}{element_list} };
 
+    if ($args{all}) {
+        my @res = grep { $self->{level}{$_} ne 'hidden' } @element_list;
+        return wantarray ? @res : "@res";
+    }
+
     # this is a bit convoluted, but the order of the returned element
     # must respect the order of the elements declared in the model by
     # the user
@@ -541,7 +534,7 @@ sub get_element_names {
         }
     }
 
-    $logger->debug("got @result");
+    $logger->trace("got @result");
 
     return wantarray ? @result : join( ' ', @result );
 }
@@ -690,11 +683,10 @@ sub fetch_element {
         );
     }
 
-    if (    $self->{status}{$element_name} eq 'deprecated'
-        and $check ne 'no' ) {
-
+    # do not warn when when is skip or "no"
+    if ($self->{status}{$element_name} eq 'deprecated' and $check eq 'yes' ) {
         # FIXME elaborate more ? or include parameter description ??
-        warn "Element '$element_name' of node '", $self->name, "' is deprecated\n";
+        warn "Element '$element_name' of node '", $self->name, "' is deprecated\n" if $check eq 'yes';
 
         # this will also force a rewrite of the file even if no other
         # semantic change was done
@@ -755,9 +747,14 @@ sub is_element_available {
 
     # force the warp to be done (if possible) so the catalog name
     # is updated
+    # retrieve element (and auto-vivify if needed)
     my $element = $self->fetch_element(
         name          => $elt_name,
-        check         => 'no',
+        # check => 'no' causes problem because elements below (when
+        # loaded by another backend also below) are initialised with
+        # check 'no'. Deprecated elements are loaded but changes are
+        # not notified because of check/no.
+        check => 'skip',
         accept_hidden => 1
     );
 
@@ -817,7 +814,9 @@ sub _get_accepted_data {
             my $dist = $tld->dld_best_distance($tld_arg);
             if ($dist < 3) {
                 my $best = $tld->dld_best_match($tld_arg);
-                warn "Warning: $name is confusingly close to $best (edit distance is $dist). Is there a typo ?\n";
+                warn "Warning: ".$self->location
+                    ." '$name' is confusingly close to '$best' (edit distance is $dist)."
+                    ." Is there a typo ?\n";
             }
 
         }
@@ -887,7 +886,7 @@ sub get {
     $path =~ s!^/!!;
     return $self unless length($path);
     my ( $item, $new_path ) = split m!/!, $path, 2;
-    $logger->debug("get: path $path, item $item");
+    $logger->trace("get: path $path, item $item");
     my $elt = $self->fetch_element( name => $item, %args );
 
     return unless defined $elt;
@@ -954,7 +953,7 @@ sub load_data {
         "Node load_data (",
         $self->location,
         ") will load elt ",
-        join( ' ', keys %$perl_data ) );
+        join( ' ', sort keys %$perl_data ) );
 
     # data must be loaded according to the element order defined by
     # the model. This will not load not yet accepted parameters
@@ -1007,7 +1006,7 @@ sub load_data {
             #TODO: annotations
             my $obj = $self->fetch_element( name => $elt, check => $check );
             next unless $obj;    # in cas of known but unavailable elements
-            $logger->debug("Node load_data: accepting element $elt");
+            $logger->info("Node load_data: accepting element $elt");
             $obj->load_data( %args, data => delete $perl_data->{$elt} ) if defined $obj;
         }
     }
@@ -1026,14 +1025,16 @@ sub load_data {
 
 sub dump_tree {
     my $self = shift;
-    $self->init;
+    my %args = @_;
+    $self->init();
     my $dumper = Config::Model::Dumper->new;
     $dumper->dump_tree( node => $self, @_ );
 }
 
 sub migrate {
     my $self = shift;
-    $self->init;
+    my %args = @_;
+    $self->init();
     Config::Model::Dumper->new->dump_tree( node => $self, mode => 'full', @_ );
 
     return $self->needs_save;
@@ -1041,14 +1042,16 @@ sub migrate {
 
 sub dump_annotations_as_pod {
     my $self = shift;
-    $self->init;
+    my %args = @_;
+    $self->init();
     my $dumper = Config::Model::DumpAsData->new;
     $dumper->dump_annotations_as_pod( node => $self, @_ );
 }
 
 sub describe {
     my $self = shift;
-    $self->init;
+    my %args = @_;
+    $self->init();
 
     my $descriptor = Config::Model::Describe->new;
     $descriptor->describe( node => $self, @_ );
@@ -1056,14 +1059,16 @@ sub describe {
 
 sub report {
     my $self = shift;
-    $self->init;
+    my %args = @_;
+    $self->init();
     my $reporter = Config::Model::Report->new;
     $reporter->report( node => $self );
 }
 
 sub audit {
     my $self = shift;
-    $self->init;
+    my %args = @_;
+    $self->init();
     my $reporter = Config::Model::Report->new;
     $reporter->report( node => $self, audit => 1 );
 }
@@ -1078,6 +1083,8 @@ sub copy_from {
     $self->load( step => $dump, check => $check );
 }
 
+# TODO: need Pod::Text attribute -> move that to a role ?
+# to translate Pod description to plain text when help is displayed
 sub get_help {
     my $self = shift;
 
@@ -1147,7 +1154,40 @@ sub apply_fixes {
 
     $fix_logger->debug( "apply fix started from ", $self->name );
     $scan->scan_node( undef, $self );
-    $fix_logger->debug("apply fix done");
+    $fix_logger->trace("apply fix done");
+}
+
+sub deep_check {
+    my $self = shift;
+    my %args = @_;
+
+    $deep_check_logger->trace("called on ".$self->name);
+
+    # no deep_check defined (yet). Note that value check is done when
+    # storing value (even during initial load, so there's no need to
+    # force a check.
+    my $check_leaf = sub { };
+
+    my $check_id = sub {
+        my ( $scanner, $data_r, $node, $element, @keys ) = @_;
+
+        $deep_check_logger->trace( "deep check called on from ", $node->name, " elt $element  keys @keys" );
+        return unless @keys;
+        $node->fetch_element($element)->deep_check;
+
+    };
+
+    my $scan = Config::Model::ObjTreeScanner->new(
+        hash_element_hook => $check_id,
+        list_element_hook => $check_id,
+        leaf_cb         => $check_leaf,
+        auto_vivify     => $args{auto_vivify},
+        check           => 'no',
+    );
+
+    $deep_check_logger->debug( "deep check started from ", $self->name );
+    $scan->scan_node( undef, $self );
+    $deep_check_logger->trace("deep check done");
 }
 
 __PACKAGE__->meta->make_immutable;
@@ -1168,7 +1208,7 @@ Config::Model::Node - Class for configuration tree node
 
 =head1 VERSION
 
-version 2.088
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -1291,6 +1331,15 @@ Element names can be grouped to save typing:
 
 See below for details on element declaration.
 
+=item B<gist>
+
+String used to construct a summary of the content of a node. This
+parameter is used by user interface to show users the gist of the
+content of this node. This parameter has no other effect. This string
+may contain element values in the form "C<{foo} or {bar}>". When
+constructing the gist, C<{foo}> is replaced by the value of element
+C<foo>. Likewise for C<{bar}>.
+
 =item B<level>
 
 Optional C<list ref> of the elements whose level are different from
@@ -1325,9 +1374,7 @@ when generating user interfaces.
 Optional C<list ref> of element descriptions. These descriptions may be
 used when generating user interfaces.
 
-=item B<read_config>
-
-=item B<write_config>
+=item B<rw_config>
 
 =item B<config_dir>
 
@@ -1574,9 +1621,9 @@ L<Config::Model::AnyThing>
 
 See L<Config::Model::AnyThing/"index_value()">
 
-=head2 parent()
+=head2 parent
 
-See L<Config::Model::AnyThing/"parent()">
+See L<Config::Model::AnyThing/"parent">
 
 =head2 root()
 
@@ -1594,9 +1641,9 @@ configuration file.
 
 =head1 Element property management
 
-=head2 get_element_names (  ...  )
+=head2 get_element_names
 
-Return all elements names available.
+Return all available element names, including the element that were accepted.
 
 Optional parameters are:
 
@@ -1604,6 +1651,13 @@ Optional parameters are:
 
 =item *
 
+B<all>: Boolean. When set return all element names, even the hidden
+ones and does not trigger warp mechanism. Defaults to 0. This option
+should be set to 1 when this method is needed to read configuration data from a
+backend.
+
+=item *
+
 B<type>: Returns only element of requested type (e.g. C<list>,
 C<hash>, C<leaf>,...). By default return elements of any type.
 
@@ -1682,6 +1736,10 @@ element is unknown, or 0 if the element is not available (hidden).
 
 Fetch and returns the I<value> of a leaf element from a node.
 
+=head2 fetch_gist
+
+Return the gist of the node. See description of C<gist> parameter above.
+
 =head2 store_element_value ( name, value )
 
 Store a I<value> in a leaf element from a node.
@@ -1716,17 +1774,17 @@ Returns 1 if the element is known in the model.
 
 Returns 1 if the element is defined.
 
-=head2 grab(...)
+=head2 grab
 
-See L<Config::Model::AnyThing/"grab(...)">.
+See L<Config::Model::Role::Grab/grab">.
 
-=head2 grab_value(...)
+=head2 grab_value
 
-See L<Config::Model::AnyThing/"grab_value(...)">.
+See L<Config::Model::Role::Grab/grab_value">.
 
-=head2 grab_root()
+=head2 grab_root
 
-See L<Config::Model::AnyThing/"grab_root()">.
+See L<Config::Model::Role::Grab/"grab_root">.
 
 =head2 get( path => ..., mode => ... ,  check => ... , get_obj => 1|0, autoadd => 1|0)
 
@@ -1737,6 +1795,13 @@ instead of returning its value.
 
 Set a value from a directory like path.
 
+=head1 Validation
+
+=head2 deep_check
+
+Scan the tree and deep check on all elements that support this. Currently only hash or
+list element have this feature.
+
 =head1 data modification
 
 =head2 migrate
@@ -1872,7 +1937,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
