@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::SearchElement;
-$Config::Model::SearchElement::VERSION = '2.088';
+$Config::Model::SearchElement::VERSION = '2.117';
 use Log::Log4perl qw(get_logger :levels);
 use Carp;
 use strict;
@@ -50,7 +50,7 @@ sub _sniff_class {
     my $model   = $self->{model};
     my $c_model = $model->get_model($class);
 
-    $logger->debug("sniffing config class $class");
+    $logger->trace("sniffing config class $class");
 
     croak "Recursive config class $class detected, aborting..."
         if defined $found_ref->{$class};
@@ -86,7 +86,7 @@ sub _sniff_class {
             $h{$element}{next_step}{$element} = '';
         }
     }
-    $logger->debug("done sniffing config class $class");
+    $logger->trace("done sniffing config class $class");
     return \%h;
 }
 
@@ -183,7 +183,7 @@ sub next_choice {
 
     while (1) {
         $result = $self->next_step;
-        $logger->debug("next_choice: result is @$result");
+        $logger->trace("next_choice: result is @$result");
         return $result if scalar @$result != 1;
 
         $self->choose(@$result);
@@ -333,7 +333,7 @@ Config::Model::SearchElement - Search an element in a configuration model
 
 =head1 VERSION
 
-version 2.088
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -579,7 +579,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
