@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2017 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Value;
-$Config::Model::Value::VERSION = '2.106';
+$Config::Model::Value::VERSION = '2.117';
 use 5.10.1;
 
 use Mouse;
@@ -193,11 +193,11 @@ sub notify_change {
     $self->map_write_as_inplace( @args{qw/old new/} );
     $self->SUPER::notify_change( %args, value_type => $self->value_type );
 
-    # notify all warped or computed objects that depends on me
+    # shake all warped or computed objects that depends on me
     foreach my $s ( $self->get_depend_slave ) {
-        $change_logger->debug( "calling notify_change on slave ", $s->name )
+        $change_logger->debug( "calling needs_check on slave ", $s->name )
             if $change_logger->is_debug;
-        $s->notify_change( note => 'master triggered changed' );
+        $s->needs_check(1);
     }
 }
 
@@ -352,7 +352,7 @@ sub migrate_value {
     $self->{migration_done} = 1;
 
     # avoid warning when reading deprecated values
-    my $result = $self->{_migrate_from}->compute( check => 'no' );
+    my $result = $self->{_migrate_from}->compute( check => 'skip' );
 
     return undef unless defined $result;
 
@@ -409,10 +409,10 @@ sub setup_match_regexp {
     return unless defined $str;
     my $vt = $self->{value_type};
 
-    if ( $vt ne 'uniline' and $vt ne 'string' ) {
+    if ( $vt ne 'uniline' and $vt ne 'string' and $vt ne 'enum') {
         Config::Model::Exception::Model->throw(
             object => $self,
-            error  => "Can't use $what regexp with $vt, " . "expected 'uniline' or 'string'"
+            error  => "Can't use $what regexp with $vt, expected 'enum', 'uniline' or 'string'"
         );
     }
 
@@ -437,10 +437,10 @@ sub check_validation_regexp {
 
     my $vt = $self->{value_type};
 
-    if ( $vt ne 'uniline' and $vt ne 'string' ) {
+    if ( $vt ne 'uniline' and $vt ne 'string' and $vt ne 'enum') {
         Config::Model::Exception::Model->throw(
             object => $self,
-            error  => "Can't use $what regexp with $vt, " . "expected 'uniline' or 'string'"
+            error  => "Can't use $what regexp with $vt, expected 'enum', 'uniline' or 'string'"
         );
     }
 
@@ -536,9 +536,18 @@ sub set_properties {
     }
 
     map { $self->{$_} = delete $args{$_} if defined $args{$_} }
-        qw/min max mandatory replace warn replace_follow assert warn_if warn_unless
+        qw/min max mandatory warn replace_follow assert warn_if warn_unless
         write_as/;
 
+    if ($args{replace}) {
+        $self->{replace} = delete $args{replace};
+        my $old = $self->_fetch_no_check;
+        if (defined $old) {
+            my $new = $self->apply_replace($old);
+            $self->_store_value($new);
+        }
+    }
+
     $self->set_help( \%args );
     $self->set_value_type( \%args );
     $self->set_default( \%args );
@@ -947,9 +956,30 @@ sub run_code_on_value {
     }
 }
 
+# function that may be used in eval'ed code to use file in there (in
+# run_code_set_on_value and apply_fix). Using this function is
+# mandatory for tests that are done in pseudo root
+# directory. Necessary for relative path (although chdir in and out of
+# root_dir could work) and for absolute path (where chdir in and out
+# of root_dir would not work without using chroot)
+
+{
+    # val is a value object. Use this trick so eval'ed code can
+    # use file() function instead of $file->() sub ref
+    my $val ;
+    sub set_val {
+        $val = shift;
+    }
+    sub file {
+        return $val->root_path->child(shift);
+    }
+}
+
 sub run_code_set_on_value {
     my ( $self, $value_r, $apply_fix, $array, $msg, $w_info, $invert ) = @_;
 
+    $self->set_val;
+
     foreach my $label ( sort keys %$w_info ) {
         my $code = $w_info->{$label}{code};
         my $msg = $w_info->{$label}{msg} || $msg;
@@ -1030,6 +1060,8 @@ sub apply_fix {
         $fix_logger->info( $self->location . ": Applying fix '$str'" );
     }
 
+    $self->set_val;
+
     eval($fix);
     if ($@) {
         Config::Model::Exception::Model->throw(
@@ -1058,14 +1090,19 @@ sub _store_fix {
             "fix change: '" . ( $old // '<undef>' ) . "' -> '" . ( $new // '<undef>' ) . "'" );
     }
 
-    my %args = (
-        old => $old // $self->_fetch_std,
-        new => $new // $self->_fetch_std,
-        note => 'applied fix'. ( $msg ? ' for :'. $msg : ''),
-    ) ;
+    my $new_v = $new // $self->_fetch_std ;
+    my $old_v => $old // $self->_fetch_std;
+
     no warnings "uninitialized";
     # in case $old is the default value and $new is undef
-    $self->notify_change( %args ) if $args{old} ne $args{new};
+    if ($old_v ne $new_v) {
+        $self->notify_change(
+            old => $old_v,
+            new => $new_v,
+            note => 'applied fix'. ( $msg ? ' for :'. $msg : '')
+        );
+        $self->trigger_warp($new_v) if defined $new_v and $self->has_warped_slaves;
+    }
 }
 
 # read checks should be blocking
@@ -1234,8 +1271,7 @@ sub _store {
 
     my $old_value = $self->_fetch_no_check;
 
-    # FIXME: storing wrong value does not make sense
-    # we let store the value even if wrong when check is disabled
+    # let's store wrong value when check is disable (gh #15)
     if ( $ok or $check eq 'no' ) {
         $self->instance->cancel_error( $self->location );
         $self->_store_value( $value, $notify_change );
@@ -1324,22 +1360,9 @@ sub transform_value {
     $value = $self->{convert_sub}($value)
         if ( defined $self->{convert_sub} and defined $value );
 
-    if ( defined $self->{replace} ) {
-        if ( defined $self->{replace}{$value} ) {
-            $logger->debug("store replacing value $value with $self->{replace}{$value}");
-            $value = $self->{replace}{$value};
-        }
-        else {
-            foreach my $k ( keys %{ $self->{replace} } ) {
-                if ( $value =~ /^$k$/ ) {
-                    $logger->debug(
-                        "store replacing value $value (matched /$k/) with $self->{replace}{$k}");
-                    $value = $self->{replace}{$k};
-                    last;
-                }
-            }
-        }
-    }
+    # apply replace on store *before* check is done, so a bad value
+    # can be replaced with a good one
+    $value = $self->apply_replace($value) if ($self->{replace} and defined $value);
 
     # using default or computed value is normally done on fetch. Except that an undefined
     # value cannot be stored in a mandatory value. Storing undef is used when resetting a
@@ -1353,6 +1376,26 @@ sub transform_value {
     return $value;
 }
 
+sub apply_replace {
+    my ($self, $value) = @_;
+
+    if ( defined $self->{replace}{$value} ) {
+        $logger->debug("store replacing value $value with $self->{replace}{$value}");
+        $value = $self->{replace}{$value};
+    }
+    else {
+        foreach my $k ( keys %{ $self->{replace} } ) {
+            if ( $value =~ /^$k$/ ) {
+                $logger->debug(
+                    "store replacing value $value (matched /$k/) with $self->{replace}{$k}");
+                $value = $self->{replace}{$k};
+                last;
+            }
+        }
+    }
+    return $value;
+}
+
 sub check_stored_value {
     my $self = shift;
     my %args = @_;
@@ -1364,15 +1407,12 @@ sub check_stored_value {
 
     $self->needs_check(0) unless $self->has_error or $self->has_warning;
 
-    # old_warn is used to avoid warning the user several times for the
-    # same reason. We take care to clean up this hash each time this routine
-    # is run
-    my $old_warn = $self->{old_warning_hash} || {};
+    # avoid warning the user several times for the same value (at
+    # store time and then at fetch time).
     my %warn_h;
     if ( $self->has_warning and not $nowarning and not $silent ) {
         foreach my $w ( $self->all_warnings ) {
             $warn_h{$w} = 1;
-            next if $old_warn->{$w};
             my $str = defined $value ? "'$value'" : '<undef>';
             warn "Warning in '" . $self->location_short . "' value $str: $w\n";
         }
@@ -1657,6 +1697,7 @@ sub fetch {
 
         # store replaced value to trigger notify_change
         if ( defined $rep and $rep ne $value ) {
+            $logger->debug( "fetch replace_follow $value with $rep from ".$self->{replace_follow});
             $value = $self->_store_value($rep);
         }
     }
@@ -1812,7 +1853,7 @@ Config::Model::Value - Strongly typed configuration value
 
 =head1 VERSION
 
-version 2.106
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -2082,6 +2123,16 @@ The example below warns if value contaims a number:
     }
  },
 
+Any operation or check on file must be done with C<file> sub
+(otherwise tests will break). This sub returns a L<Path::Tiny>
+object that can be used to perform checks. For instance:
+
+  warn_if => {
+     warn_test => {
+         code => 'not file($_)->exists',
+         msg  => 'file $_ should exist'
+     }
+
 =item warn_unless
 
 Like C<warn_if>, but issue a warning when the given C<code> returns false.
@@ -2845,7 +2896,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2017 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
