@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Value::LayeredInclude;
-$Config::Model::Value::LayeredInclude::VERSION = '2.097';
+$Config::Model::Value::LayeredInclude::VERSION = '2.117';
 use 5.010;
 use strict;
 use warnings;
@@ -81,7 +81,7 @@ sub _check_value {
 
     # need to test that prest config file is present as the model
     # may not enforce it (when read_config auto_create is 1)
-    my $layered_file = $self->instance->read_root_dir;
+    my $layered_file = $self->instance->root_dir;
     $layered_file .= $value;
 
     my $err = $self->{error_list};
@@ -108,7 +108,7 @@ Config::Model::Value::LayeredInclude - Include a sub layer configuration
 
 =head1 VERSION
 
-version 2.097
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -164,7 +164,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
