@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::WarpedNode;
-$Config::Model::WarpedNode::VERSION = '2.097';
+$Config::Model::WarpedNode::VERSION = '2.117';
 use Mouse;
 
 use Carp qw(cluck croak);
@@ -33,7 +33,7 @@ my $logger = get_logger("Tree::Node::Warped");
 # status is not warpable either as an obsolete parameter must stay
 # obsolete
 
-my @allowed_warp_params = qw/config_class_name level/;
+my @allowed_warp_params = qw/config_class_name level gist/;
 
 has 'backup' => ( is => 'rw', isa => 'HashRef', default => sub { {}; } );
 
@@ -80,8 +80,8 @@ sub config_model {
 # Forward selected methods (See man perltootc)
 foreach my $method (
     qw/fetch_element config_class_name copy_from get_element_name
-    has_element is_element_available element_type load
-    fetch_element_value get_type get_cargo_type dump_tree
+    fetch_gist has_element is_element_available element_type load
+    fetch_element_value get_type get_cargo_type dump_tree needs_save
     describe get_help get_help_as_text children get set accept_regexp/
     ) {
     # to register new methods in package
@@ -143,7 +143,7 @@ sub set_properties {
     # mega cleanup
     map( delete $self->{$_}, @allowed_warp_params );
 
-    $logger->debug( $self->name . " set_properties called with ",
+    $logger->trace( $self->name . " set_properties called with ",
         Data::Dumper->Dump( [ \%args ], ['set_properties_args'] ) );
 
     my $config_class_name = delete $args{config_class_name};
@@ -313,7 +313,7 @@ Config::Model::WarpedNode - Node that change config class properties
 
 =head1 VERSION
 
-version 2.097
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -560,7 +560,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
