@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Warper;
-$Config::Model::Warper::VERSION = '2.088';
+$Config::Model::Warper::VERSION = '2.117';
 use Mouse;
 
 use Log::Log4perl qw(get_logger :levels);
@@ -23,7 +23,7 @@ has 'rules'  => ( is => 'ro', isa => 'ArrayRef',     required => 1 );
 has 'warped_object' => (
     is       => 'ro',
     isa      => 'Config::Model::AnyThing',
-    handles  => ['notify_change'],
+    handles  => ['needs_check'],
     weak_ref => 1,
     required => 1
 );
@@ -62,7 +62,7 @@ my $logger = get_logger("Warper");
 sub BUILD {
     my $self = shift;
 
-    $logger->debug( "Warper new: created for " . $self->name );
+    $logger->trace( "Warper new: created for " . $self->name );
     $self->check_warp_args;
 
     $self->register_to_all_warp_masters;
@@ -423,7 +423,7 @@ sub compute_bool {
     my $self = shift;
     my $expr = shift;
 
-    $logger->debug("Warper compute_bool: called for '$expr'");
+    $logger->trace("Warper compute_bool: called for '$expr'");
 
     #    my $warp_value_set = $self->_values   ;
     $logger->debug( "Warper compute_bool: data:\n",
@@ -443,7 +443,7 @@ sub compute_bool {
     }
 
     my $perl_code = join( "\n", @init_code, $expr );
-    $logger->debug("Warper compute_bool: eval code '$perl_code'");
+    $logger->trace("Warper compute_bool: eval code '$perl_code'");
 
     my $ret;
     {
@@ -482,7 +482,7 @@ sub do_warp {
         next unless $res;
         $found_bool = $bool_expr;
         $found_rule = $rule_hash{$bool_expr} || {};
-        $logger->debug(
+        $logger->trace(
             "do_warp found rule for '$bool_expr':\n",
             Data::Dumper->Dump( [$found_rule], ['found_rule'] ) );
         last;
@@ -502,7 +502,7 @@ sub do_warp {
         );
     }
 
-    $logger->debug( "do_warp: call set_parent_element_property on '",
+    $logger->trace( "do_warp: call set_parent_element_property on '",
         $self->name, "' with ", Data::Dumper->Dump( [$found_rule], ['found_rule'] ) );
 
     $self->set_parent_element_property($found_rule);
@@ -620,7 +620,7 @@ Config::Model::Warper - Warp tree properties
 
 =head1 VERSION
 
-version 2.088
+version 2.117
 
 =head1 SYNOPSIS
 
@@ -654,13 +654,13 @@ and C<rules>:
 
 =head2 Warp follow argument
 
-L<Grab string|Config::Model::AnyThing/"grab(...)"> leading to the
+L<Grab string|Config::Model::Role::Grab/grab> leading to the
 C<Config::Model::Value> or L<Config::Model::CheckList> warp master. E.g.:
 
  follow => '! tree_macro' 
 
 In case of several warp master, C<follow> is set to an array ref
-of several L<grab string|Config::Model::AnyThing/"grab(...)">:
+of several L<grab string|Config::Model::Role::Grab/grab>:
 
  follow => [ '! macro1', '- macro2' ]
 
@@ -822,7 +822,7 @@ warp master is an enumerated type)
 When a warped object is created, the constructor registers to the
 warp masters. The warp master are found by using the special string
 passed to the C<follow> parameter. As explained in 
-L<grab method|Config::Model::AnyThing/"grab(...)">,
+L<grab method|Config::Model::Role::Grab/grab>,
 the string provides the location of the warp master in the
 configuration tree using a symbolic form. 
 
@@ -874,7 +874,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2018 by Dominique Dumont.
 
 This is free software, licensed under:
 
