@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -34,7 +34,7 @@
           'maybe',
           'yes'
         ],
-        'description' => 'encrypt popcon submission. Eventually, this feature wil be enabled by default.',
+        'description' => 'encrypt popcon submission. Eventually, this feature will be enabled by default.',
         'help' => {
           'maybe' => 'encrypt if gpg is available',
           'yes' => 'try to encrypt and fail if gpg is not available'
@@ -105,13 +105,11 @@ This key was generated automatically so you should normally just leave it alone.
     ],
     'license' => 'LGPL2',
     'name' => 'PopCon',
-    'read_config' => [
-      {
-        'backend' => 'ShellVar',
-        'config_dir' => '/etc',
-        'file' => 'popularity-contest.conf'
-      }
-    ]
+    'rw_config' => {
+      'backend' => 'ShellVar',
+      'config_dir' => '/etc',
+      'file' => 'popularity-contest.conf'
+    }
   }
 ]
 ;
