@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2013 by Dominique Dumont.
+# This software is Copyright (c) 2005-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -40,8 +40,15 @@ $model_to_test = "Shelly";
         check => [
             foo => 'ok',
             bar => "with space"
-        ]
+        ],
     },
+    {
+        # data is written in file not using canonical order
+        name => 'keep-order',
+        file_contents_like => {
+            "/etc/foo.conf" => [ qr/bar="with space"\nfoo="ok"/m ] ,
+        }
+    }
 );
 
 1;
